#!/bin/bash

# Enable debug mode so that CakePHP will create missing folders
# https://github.com/it-novum/openITCOCKPIT/issues/1446
# https://github.com/cakephp/migrations/issues/565
export OITC_DEBUG=1

# Stop phpnsta-client after upgrading from V3 to V4
if systemctl is-active phpnsta-client.service; then
    systemctl stop phpnsta-client.service
fi

APPDIR="/opt/openitc/frontend"

INIFILE=/opt/openitc/etc/mysql/mysql.cnf
DUMPINIFILE=/opt/openitc/etc/mysql/dump.cnf
BASHCONF=/opt/openitc/etc/mysql/bash.conf

DEBIANCNF=/etc/mysql/debian.cnf

MYSQL_USER="openitcockpit"
MYSQL_DATABASE="openitcockpit"
MYSQL_PASSWORD=$(pwgen -s -1 16)

PHPVersion=$(php -r "echo substr(PHP_VERSION, 0, 3);")

OSVERSION=$(grep VERSION_CODENAME /etc/os-release | cut -d= -f2)
OS_BASE="debian"

if [[ -f "/etc/redhat-release" ]]; then
    echo "Detected RedHat based operating system."
    OS_BASE="RHEL"
    OSVERSION=$(source /etc/os-release && echo $VERSION_ID | cut -d. -f1) # e.g. 8
fi

if [[ "$OS_BASE" == "RHEL" ]]; then
    echo "Make RedHat look more like it's Debian ;)"

    if [[ "$(LANG=c getenforce)" == "Enforcing" ]]; then
        /usr/sbin/semanage permissive -a httpd_t
    fi

    # sudo - allow root to use sudo
    # sudo is needed by the "oitc" command
    usermod -aG wheel root
    echo "root    ALL=(ALL:ALL) ALL" > /etc/sudoers.d/openitcockpit
    chmod 0440 /etc/sudoers.d/openitcockpit

    # php-fpm
    mkdir -p "/etc/php/${PHPVersion}/fpm"
    mkdir -p /run/php

    mkdir -p /etc/systemd/system/php-fpm.service.d/
    echo "[Service]" > /etc/systemd/system/php-fpm.service.d/override.conf
    echo "# Create folder for openITCOCKPIT default php-fpm socket path" >> /etc/systemd/system/php-fpm.service.d/override.conf
    echo "ExecStartPre=-mkdir -p /run/php" >> /etc/systemd/system/php-fpm.service.d/override.conf


    # Link RedHat config to where they are on Debian
    ln -s /etc/php-fpm.conf "/etc/php/${PHPVersion}/fpm/php-fpm.conf"
    ln -s /etc/php.d "/etc/php/${PHPVersion}/fpm/conf.d"
    ln -s /etc/php-fpm.d "/etc/php/${PHPVersion}/fpm/pool.d"

    # nginx
    if [[ -f "/etc/nginx/nginx.conf" ]]; then
        echo "Move /etc/nginx/nginx.conf to /etc/nginx/nginx.conf.orig"
        mv /etc/nginx/nginx.conf /etc/nginx/nginx.conf.orig
    fi
    mkdir -p /etc/nginx/sites-enabled
    mkdir -p /var/lib/nginx/tmp
    chown www-data:root /var/lib/nginx -R
    chown www-data:root /var/lib/nginx/tmp -R
    cp ${APPDIR}/system/nginx/nginx.rhel${OSVERSION}.conf /etc/nginx/nginx.conf

    # myqsl
    mkdir -p /etc/mysql
    ln -s /etc/my.cnf.d /etc/mysql/conf.d

    # create snakeoil ssl certificate if no exists
    mkdir -p /etc/ssl/private
    if [[ ! -f "/etc/ssl/private/ssl-cert-snakeoil.key" ]]; then
        openssl req -x509 -nodes -days 365 -newkey rsa:2048 -keyout /etc/ssl/private/ssl-cert-snakeoil.key -out /etc/ssl/certs/ssl-cert-snakeoil.pem -subj "/C=US/ST=Denial/L=Springfield/O=Dis/CN=localhost"
        openssl dhparam -out /etc/ssl/certs/dhparam.pem 2048
        cat /etc/ssl/certs/dhparam.pem | tee -a /etc/ssl/certs/ssl-cert-snakeoil.pem

        chown root:nagios /etc/ssl/private/ssl-cert-snakeoil.key
        chown root:nagios /etc/ssl/certs/ssl-cert-snakeoil.pem
        chmod 640 /etc/ssl/private/ssl-cert-snakeoil.key
    fi

    systemctl daemon-reload

    systemctl enable\
     nginx.service\
     gearmand.service\
     mysqld.service\
     redis.service\
     docker.service\
     php-fpm.service

    systemctl start\
    nginx.service\
     gearmand.service\
     mysqld.service\
     redis.service\
     docker.service\
     php-fpm.service

fi # End RHEL section

ENABLE_WEB_INTERFACE="false"
if [[ -f "/opt/openitc/etc/frontend/enable_web_interface" ]]; then
    ENABLE_WEB_INTERFACE="true"
fi

echo "Copy required system files"
mkdir -p /opt/openitc/nagios/etc/config
mkdir -p /opt/openitc/etc/nagios /opt/openitc/etc/statusengine /opt/openitc/etc/nagios/nagios.cfg.d
mkdir -p /opt/openitc/etc/{mysql,frontend,nagios,nsta,statusengine}
mkdir -p /opt/openitc/docker/{container,images}
mkdir -p /opt/openitc/docker/container/graphing
mkdir -p /opt/openitc/etc/mod_gearman
mkdir -p /opt/openitc/logs/mod_gearman
chown nagios:nagios /opt/openitc/logs/mod_gearman

rsync -K -a ${APPDIR}/system/etc/. /etc/ # we use rsync because the destination can be a symlink on RHEL
cp -r ${APPDIR}/system/opt/. /opt/
cp -r ${APPDIR}/system/lib/. /lib/
cp -r ${APPDIR}/system/usr/. /usr/
rsync -K -a ${APPDIR}/system/fpm/. /etc/php/${PHPVersion}/fpm/
cp ${APPDIR}/system/nginx/ssl_options_$OSVERSION /etc/nginx/openitc/ssl_options.conf

if [[ ! -f "/etc/nginx/openitc/ssl_cert.conf" ]]; then
    cp ${APPDIR}/system/nginx/ssl_cert.conf /etc/nginx/openitc/ssl_cert.conf
fi

cp -r ${APPDIR}/system/Statusengine3/mysql.php /opt/openitc/statusengine3/worker/lib/mysql.php

if [[ ! -f "/etc/nginx/openitc/custom.conf" ]]; then
    echo "# This file will NOT be overwritten during an update" >> /etc/nginx/openitc/custom.conf
fi

mkdir -p /opt/openitc/agent
chown nagios:nagios /opt/openitc/agent
mkdir -p /opt/openitc/receiver/etc
chown nagios:nagios /opt/openitc/receiver/etc


ln -sf /opt/openitc/etc/nagios/nagios.cfg /opt/openitc/nagios/etc/nagios.cfg
chown nagios:www-data /opt/openitc/nagios/etc -R
chown nagios:www-data /opt/openitc/etc/nagios -R

chmod 0400 /etc/sudoers.d/nagios
chown root:root /etc/sudoers.d -R
chmod +x /opt/openitc/nagios/bin/restart-monitoring.sh

mkdir -p /opt/openitc/logs/frontend
ln -s /opt/openitc/logs/frontend/ /opt/openitc/frontend/logs

chown nagios:www-data /opt/openitc/logs/frontend
chmod 770 /opt/openitc/logs/frontend

mkdir -p /opt/openitc/frontend/tmp
chown www-data:www-data /opt/openitc/frontend/tmp


mkdir -p /opt/openitc/logs/frontend/nagios
chown www-data:www-data /opt/openitc/logs/frontend
chown nagios:nagios /opt/openitc/logs/frontend/nagios
chmod 775 /opt/openitc/logs/frontend
chmod 775 /opt/openitc/logs/frontend/nagios

mkdir -p /opt/openitc/logs/nagios/archives
chown nagios:www-data /opt/openitc/logs/nagios /opt/openitc/logs/nagios/archives
chmod 775 /opt/openitc/logs/nagios /opt/openitc/logs/nagios/archives

mkdir -p /opt/openitc/frontend/tmp/nagios
chown www-data:www-data /opt/openitc/frontend/tmp
chown nagios:nagios /opt/openitc/frontend/tmp/nagios

mkdir -p /opt/openitc/frontend/tmp/cache/
chown www-data:www-data /opt/openitc/frontend/tmp/cache -R

chmod u+s /opt/openitc/nagios/libexec/check_icmp
chmod u+s /opt/openitc/nagios/libexec/check_dhcp

mkdir -p /opt/openitc/frontend/webroot/img/userimages
chown www-data:www-data /opt/openitc/frontend/webroot/img/userimages -R

if [[ "$ENABLE_WEB_INTERFACE" == "true" ]]; then
    ln -s /etc/nginx/sites-available/openitc /etc/nginx/sites-enabled/openitc
fi
rm -f /etc/nginx/sites-enabled/default

chmod +x /usr/bin/oitc

if [[ -d /opt/openitc/check_mk ]]; then
  chown nagios:nagios /opt/openitc/check_mk -R
fi

if [[ ! -f "$INIFILE" ]]; then
    echo "Create local MySQL configuration and database"

    if [[ "$OS_BASE" == "RHEL" ]]; then
        systemctl restart mysqld.service
    else
        # Restart MySQL to load openitcockpit.cnf to disable MySQL Binary Logs on Ubuntu Focal
        systemctl restart mysql.service
    fi
    echo "Waiting 3 seconds for MySQL / MariaDB..."
    sleep 3

    if [[ -f "$DEBIANCNF" ]]; then
        echo "Detected Debian based distribution"

        # set sql_mode to enable group by like it was in the good old times :)
        # Also this will truncate too long strings: Data too long for column 'check_command' ...
        mysql --defaults-extra-file=${DEBIANCNF} -e "SET GLOBAL sql_mode = '';"

        if [[ "$OS_BASE" == "RHEL" ]]; then
            # Set Password policy to LOW on RHEL systems with MySQL 8
            # https://dev.mysql.com/doc/refman/8.0/en/validate-password-options-variables.html#sysvar_validate_password.policy
            mysql --defaults-extra-file=${DEBIANCNF} -e "SET GLOBAL validate_password.policy=LOW;" -B
        fi

        MYSQL_OITC_USER_EXISTS=$(mysql "--defaults-extra-file=${DEBIANCNF}" -e "SELECT EXISTS(SELECT 1 FROM mysql.user WHERE user = 'openitcockpit')" -B -s 2>/dev/null)

        if [[ "$MYSQL_OITC_USER_EXISTS" -eq "1" ]]; then
            mysql --defaults-extra-file=${DEBIANCNF} -e "ALTER USER '${MYSQL_DATABASE}'@'localhost' IDENTIFIED BY '${MYSQL_PASSWORD}';" -B
        else
            mysql --defaults-extra-file=${DEBIANCNF} -e "CREATE USER '${MYSQL_DATABASE}'@'localhost' IDENTIFIED BY '${MYSQL_PASSWORD}';" -B
        fi

        mysql --defaults-extra-file=${DEBIANCNF} -e "CREATE DATABASE IF NOT EXISTS \`${MYSQL_DATABASE}\` DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci;" -B
        mysql --defaults-extra-file=${DEBIANCNF} -e "GRANT ALL PRIVILEGES ON \`${MYSQL_DATABASE}\`.* TO '${MYSQL_DATABASE}'@'localhost';" -B

        echo "; Automatically generated for openITCOCKPIT scripts. DO NOT TOUCH!" >$INIFILE
        echo "[client]" >>$INIFILE
        echo "database = ${MYSQL_DATABASE}" >>$INIFILE
        echo "host = localhost" >>$INIFILE
        echo "user = ${MYSQL_USER}" >>$INIFILE
        echo "password = ${MYSQL_PASSWORD}" >>$INIFILE
        echo "port = 3306" >>$INIFILE

        echo "; Automatically generated for mysqldump. DO NOT TOUCH!" >$DUMPINIFILE
        echo "[client]" >>$DUMPINIFILE
        echo "host     = localhost" >>$DUMPINIFILE
        echo "user     = ${MYSQL_USER}" >>$DUMPINIFILE
        echo "password = ${MYSQL_PASSWORD}" >>$DUMPINIFILE
        echo "port     = 3306" >>$DUMPINIFILE

        echo "dbc_dbuser='${MYSQL_USER}'" >$BASHCONF
        echo "dbc_dbpass='${MYSQL_PASSWORD}'" >>$BASHCONF
        echo "dbc_dbserver='127.0.0.1'" >>$BASHCONF
        echo "dbc_dbport='3306'" >>$BASHCONF
        echo "dbc_dbname='${MYSQL_DATABASE}'" >>$BASHCONF

        chown www-data:nagios $INIFILE
        chmod 660 $INIFILE

        chown www-data:nagios $DUMPINIFILE
        chmod 660 $DUMPINIFILE

        chown www-data:nagios $BASHCONF
        chmod 660 $BASHCONF

    else
        echo "Unsupported distribution or $DEBIANCNF is missing!"
        exit 1
    fi

fi

echo "Checking for interface.yml"
if [[ ! -f "${APPDIR}/config/interface.yml" ]]; then
  echo "Copy interface.defaults.yml to interface.yml"
  cp ${APPDIR}/config/interface.defaults.yml ${APPDIR}/config/interface.yml
fi


echo "---------------------------------------------------------------"
echo "Running openITCOCKPIT Satellite database migration"
oitc migrations migrate

oitc migrations seed

echo "Running openITCOCKPIT Satellite Module database migration/s"
for PLUGIN in $(ls -1 "${APPDIR}/plugins"); do
  if [[ "$PLUGIN" == *Module ]]; then
    if [[ -d "${APPDIR}/plugins/${PLUGIN}/config/Migrations" ]]; then
      echo "Running openITCOCKPIT ${PLUGIN} database migration"
      oitc migrations migrate -p "${PLUGIN}"
    fi

    if [[ -d "${APPDIR}/plugins/${PLUGIN}/config/Seeds" ]]; then
      num_files=$(find "${APPDIR}/plugins/${PLUGIN}/config/Seeds" -mindepth 1 -iname "*.php" -type f | wc -l)
      if [[ "$num_files" -gt 0 ]]; then
        echo "Importing default records for ${PLUGIN} into database"
        oitc migrations seed -p "${PLUGIN}"
      fi
    fi

  fi
done

DBSERVER=$(awk '$1 == "host" { print }' "$INIFILE" |cut -d= -f2 | sed 's/^\s*//' | sed 's/\s*$//' | sed 's_/_\\/_g')
DBUSER=$(awk '$1 == "user" { print }' "$INIFILE" |cut -d= -f2 | sed 's/^\s*//' | sed 's/\s*$//' | sed 's_/_\\/_g')
DBPW=$(awk '$1 == "password" { print }' "$INIFILE" |cut -d= -f2 | sed 's/^\s*//' | sed 's/\s*$//' | sed 's_/_\\/_g')
DBNAME=$(awk '$1 == "database" { print }' "$INIFILE" |cut -d= -f2 | sed 's/^\s*//' | sed 's/\s*$//' | sed 's_/_\\/_g')
DBPORT=$(awk '$1 == "port" { print }' "$INIFILE" |cut -d= -f2 | sed 's/^\s*//' | sed 's/\s*$//' | sed 's_/_\\/_g')

oitc config_generator --statusengine-worker
oitc config_generator --statusengine-broker
oitc config_generator --carbon-stack

echo "---------------------------------------------------------------"
echo "Load Statusengine DB Schema"

chmod +x /opt/openitc/statusengine3/worker/bin/Console.php
chmod +x /opt/openitc/statusengine3/worker/bin/StatusengineWorker.php

/opt/openitc/statusengine3/worker/bin/Console.php database --update


PERFDATA_BACKEND=$(oitc config_generator --get-perfdata-backend)

echo "---------------------------------------------------------------"
echo "Convert MySQL Tables from utf8_general_ci to utf8mb4_general_ci..."

mysql --defaults-extra-file=${INIFILE} -e "ALTER DATABASE ${MYSQL_DATABASE} CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci;"

mysql --defaults-extra-file=${INIFILE} --batch --skip-column-names -e "SELECT TABLE_NAME FROM \`information_schema\`.\`TABLES\` WHERE \`TABLE_SCHEMA\`='${MYSQL_DATABASE}' AND \`TABLE_NAME\` NOT LIKE 'nagios_%' AND \`TABLE_COLLATION\`='utf8_general_ci'" | while read TABLE_NAME; do
    echo "ALTER TABLE \`${TABLE_NAME}\` CONVERT TO CHARACTER SET utf8mb4; ✔"
    mysql --defaults-extra-file=${INIFILE} -e "ALTER TABLE \`${TABLE_NAME}\` CONVERT TO CHARACTER SET utf8mb4;"
done
echo ""

if [ ! -f /opt/openitc/etc/mod_gearman/secret.file ]; then
    echo "Generate new shared secret for Mod-Gearman"
    MG_KEY=$(php -r "echo bin2hex(openssl_random_pseudo_bytes(16, \$cstrong));")
    echo $MG_KEY > /opt/openitc/etc/mod_gearman/secret.file
fi
chown nagios:nagios /opt/openitc/etc/mod_gearman/secret.file
chmod 400 /opt/openitc/etc/mod_gearman/secret.file

#Compress and minify javascript files
oitc compress

#Acc ALC dependencies config for itc core
echo "---------------------------------------------------------------"
echo "Scan for new user permissions. This will take a while..."
oitc Acl.acl_extras aco_sync

#Set default permissions, check for always allowed permissions and dependencies
oitc roles --enable-defaults --admin

echo "Flush redis cache"
redis-cli FLUSHALL
echo ""

echo "---------------------------------------------------------------"
echo "Restart services oitc_cmd.service statusengine.service nagios.service statusengine_cleanup.timer"

if getent group ssl-cert &>/dev/null; then
    if [ -z "$(groups nagios | grep ssl-cert)" ]; then
        usermod -aG ssl-cert nagios
    fi
fi

systemctl enable\
 oitc_cmd.service \
 statusengine_cleanup.timer

if [[ "$ENABLE_WEB_INTERFACE" == "true" ]]; then
    systemctl enable statusengine.service
fi

if [[ $PERFDATA_BACKEND == "whisper" ]]; then
  systemctl enable openitcockpit-graphing.service
fi

systemctl restart\
 oitc_cmd.service \
 statusengine_cleanup.timer

if [[ "$ENABLE_WEB_INTERFACE" == "true" ]]; then
    systemctl restart statusengine.service
fi

if [[ $PERFDATA_BACKEND == "whisper" ]]; then
  echo "---------------------------------------------------------------"
  echo "Restart services openitcockpit-graphing.service"
  systemctl restart openitcockpit-graphing.service
fi

# Restart services if they are running
# Nagios/Naemon is aliased, so it works for both
for srv in nginx.service nsta.service; do
  if systemctl is-active --quiet $srv; then
    echo "Restart service: $srv"
    systemctl enable $srv
    systemctl restart $srv
  fi
done

# prevent error message after installation
if [[ -f /opt/openitc/nagios/etc/resource.cfg ]]; then
    if [[ -f /lib/systemd/system/nagios.service ]]; then
      systemctl enable nagios
      systemctl restart nagios
    fi

    if [[ -f /lib/systemd/system/naemon.service ]]; then
      systemctl enable naemon
      systemctl restart naemon
    fi
fi

if [[ -f /lib/systemd/system/openitcockpit-checkmk.service ]]; then
    echo "Restart Docker Container of Checkmk"
    systemctl enable openitcockpit-checkmk.service
    systemctl restart openitcockpit-checkmk.service
fi

# Restart services if they exists
# Mod_Gearman workers are optional because they could be offloaded to a different host
for srv in mod-gearman-worker.service; do
  if systemctl is-enabled --quiet $srv &>/dev/null; then
    echo "Restart service: $srv"
    systemctl restart $srv
  fi
done

PHPVersion=$(php -r "echo substr(PHP_VERSION, 0, 3);")
echo "Detected PHP Version: ${PHPVersion} try to restart php-fpm"

set +e
systemctl is-enabled --quiet php${PHPVersion}-fpm.service &>/dev/null
RC=$?
if [ $RC -eq 0 ]; then
  #Is it php7.3-fpm-service ?
  systemctl restart php${PHPVersion}-fpm.service
else
  # Is it just php-fpm.service?
  systemctl is-enabled --quiet php-fpm.service
  RC=$?
  if [ $RC -eq 0 ]; then
    systemctl restart php-fpm.service
  else
    echo "ERROR: could not detect php-fpm systemd service file. You need to restart php-fpm manualy"
  fi
fi
set -e

echo "Cleanup old Docker images"
if systemctl is-active --quiet docker.service; then
    docker image prune --force
    echo "Docker cleanup complete"
else
    echo "Docker is NOT Running";
    echo "Please start Docker and run the following command manually"
    echo "docker image prune -a --force"
fi
