<?php
/**
 * Routes configuration
 *
 * In this file, you set up routes to your controllers and their actions.
 * Routes are very important mechanism that allows you to freely connect
 * different URLs to chosen controllers and their actions (functions).
 *
 * CakePHP(tm) : Rapid Development Framework (http://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @link          http://cakephp.org CakePHP(tm) Project
 * @license       http://www.opensource.org/licenses/mit-license.php MIT License
 */

use Cake\Http\Middleware\CsrfProtectionMiddleware;
use Cake\Routing\Route\InflectedRoute;
use Cake\Routing\RouteBuilder;

/**
 * The default class to use for all routes
 *
 * The following route classes are supplied with CakePHP and are appropriate
 * to set as the default:
 *
 * - Route
 * - InflectedRoute
 * - DashedRoute
 *
 * If no call is made to `Router::defaultRouteClass()`, the class used is
 * `Route` (`Cake\Routing\Route\Route`)
 *
 * Note that `Route` does not do any inflections on URLs which will result in
 * inconsistently cased URLs when used with `:plugin`, `:controller` and
 * `:action` markers.
 *
 */
/** @var \Cake\Routing\RouteBuilder $routes */

$routes->setRouteClass(InflectedRoute::class);

$routes->setExtensions(['json', 'html', 'pdf', 'png', 'zip']);

$routes->scope('/', function (RouteBuilder $builder) {
    // Register scoped middleware for in scopes.
    $builder->registerMiddleware('csrf', new CsrfProtectionMiddleware([
        'httponly' => true,
    ]));
    if (
        (!isset($_SERVER['HTTP_AUTHORIZATION']) || (isset($_SERVER['HTTP_AUTHORIZATION']) && strpos($_SERVER['HTTP_AUTHORIZATION'], 'X-OITC-API') === false)) &&
        (!isset($_SERVER['QUERY_STRING']) || (isset($_SERVER['QUERY_STRING']) && strpos($_SERVER['QUERY_STRING'], 'apikey') === false))) {

        $builder->applyMiddleware('csrf');
    }
    /*
     * Fireup the AngularJS application
     */
    $builder->connect('/', ['controller' => 'Pages', 'action' => 'index']);

    /*
     * Backwards compatibility because Firefox has cached /ng redirect forever
     */
    $builder->connect('/ng', ['controller' => 'Pages', 'action' => 'index']);

    /*
 * Connect catchall routes for all controllers.
 *
 * The `fallbacks` method is a shortcut for
 *
 * ```
 * $builder->connect('/:controller', ['action' => 'index']);
 * $builder->connect('/:controller/:action/*', []);
 * ```
 *
 * You can remove these routes once you've connected the
 * routes you want in your application.
 */
    $builder->fallbacks();

});
