import type { Container } from "../Container";
import type { Engine } from "../../engine";
import type { IContainerPlugin } from "../Interfaces/IContainerPlugin";
import type { IInteractor } from "../Interfaces/IInteractor";
import type { IMovePathGenerator } from "../Interfaces/IMovePathGenerator";
import type { IOptions } from "../../Options/Interfaces/IOptions";
import type { IParticleMover } from "../Interfaces/IParticlesMover";
import type { IParticleUpdater } from "../Interfaces/IParticleUpdater";
import type { IPlugin } from "../Interfaces/IPlugin";
import type { IShapeDrawer } from "../Interfaces/IShapeDrawer";
import type { Options } from "../../Options/Classes/Options";
import type { RecursivePartial } from "../../Types/RecursivePartial";
declare type InteractorInitializer = (container: Container) => IInteractor;
declare type MoverInitializer = (container: Container) => IParticleMover;
declare type UpdaterInitializer = (container: Container) => IParticleUpdater;
export declare class Plugins {
    #private;
    readonly plugins: IPlugin[];
    readonly interactorsInitializers: Map<string, InteractorInitializer>;
    readonly moversInitializers: Map<string, MoverInitializer>;
    readonly updatersInitializers: Map<string, UpdaterInitializer>;
    readonly interactors: Map<Container, IInteractor[]>;
    readonly movers: Map<Container, IParticleMover[]>;
    readonly updaters: Map<Container, IParticleUpdater[]>;
    readonly presets: Map<string, RecursivePartial<IOptions>>;
    readonly drawers: Map<string, IShapeDrawer>;
    readonly pathGenerators: Map<string, IMovePathGenerator>;
    constructor(engine: Engine);
    getPlugin(plugin: string): IPlugin | undefined;
    addPlugin(plugin: IPlugin): void;
    getAvailablePlugins(container: Container): Map<string, IContainerPlugin>;
    loadOptions(options: Options, sourceOptions: RecursivePartial<IOptions>): void;
    getPreset(preset: string): RecursivePartial<IOptions> | undefined;
    addPreset(presetKey: string, options: RecursivePartial<IOptions>, override?: boolean): void;
    getShapeDrawer(type: string): IShapeDrawer | undefined;
    addShapeDrawer(type: string, drawer: IShapeDrawer): void;
    getSupportedShapes(): IterableIterator<string>;
    getPathGenerator(type: string): IMovePathGenerator | undefined;
    addPathGenerator(type: string, pathGenerator: IMovePathGenerator): void;
    getInteractors(container: Container, force?: boolean): IInteractor[];
    addInteractor(name: string, initInteractor: (container: Container) => IInteractor): void;
    getUpdaters(container: Container, force?: boolean): IParticleUpdater[];
    addParticleUpdater(name: string, initUpdater: (container: Container) => IParticleUpdater): void;
    getMovers(container: Container, force?: boolean): IParticleMover[];
    addParticleMover(name: string, initMover: (container: Container) => IParticleMover): void;
}
export {};
