<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace MapModule\Controller;

use App\Model\Table\HostgroupsTable;
use App\Model\Table\HoststatusTable;
use App\Model\Table\ServicegroupsTable;
use App\Model\Table\ServicestatusTable;
use Cake\Datasource\Exception\RecordNotFoundException;
use Cake\Http\Exception\BadRequestException;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use Exception;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use MapModule\Model\Table\MapitemsTable;
use MapModule\Model\Table\MapsTable;
use MapModule\Model\Table\MapsummaryitemsTable;
use RuntimeException;


/**
 * Class MapeditorsController
 * @package MapModule\Controller
 */
class MapeditorsController extends AppController {

    /**
     * @param null $id
     */
    public function view($id = null) {
        if (!$this->isApiRequest() && $id === null) {
            //Only ship html template
            return;
        }

        /** @var MapsTable $MapsTable */
        $MapsTable = TableRegistry::getTableLocator()->get('MapModule.Maps');
        $id = (int)$id;

        if (!$MapsTable->existsById($id)) {
            throw new NotFoundException();
        }
        $map = $MapsTable->get($id, [
            'contain' => [
                'Mapgadgets',
                'Mapicons',
                'Mapitems',
                'Maplines',
                'Maptexts',
                'Mapsummaryitems'
            ]
        ])->toArray();

        $acl = [
            'hosts'         => [
                'browser' => isset($this->PERMISSIONS['hosts']['browser']),
                'index'   => isset($this->PERMISSIONS['hosts']['index'])
            ],
            'services'      => [
                'browser' => isset($this->PERMISSIONS['services']['browser']),
                'index'   => isset($this->PERMISSIONS['services']['index'])

            ],
            'hostgroups'    => [
                'extended' => isset($this->PERMISSIONS['hostgroups']['extended'])
            ],
            'servicegroups' => [
                'extended' => isset($this->PERMISSIONS['servicegroups']['extended'])
            ]
        ];

        $this->set('map', $map);
        $this->set('ACL', $acl);

        $this->viewBuilder()->setOption('serialize', ['map', 'ACL']);
    }

    /**
     * @throws Exception
     */
    public function mapitem() {
        if (!$this->isApiRequest()) {
            return;
        }
        $objectId = $this->request->getQuery('objectId');
        if (!$objectId) {
            throw new RuntimeException('Invalid object id');
        }
        $mapId = (int)$this->request->getQuery('mapId');
        if ($mapId <= 0) {
            throw new RuntimeException('Invalid map id');
        }

        try {
            $data = $this->_mapitem(
                $objectId,
                $mapId,
                $this->request->getQuery('type'),
                $this->request->getQuery('includeServiceOutput')
            );
        } catch (Exception $e) {
            throw $e;
        }

        $this->set('type', $this->request->getQuery('type'));
        $this->set('data', $data['data']);
        $this->viewBuilder()->setOption('serialize', ['type', 'data']);
    }

    /**
     * @throws Exception
     */
    public function mapitemMulti() {
        if (!$this->isApiRequest()) {
            return;
        }

        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        $items = $this->request->getData('items', []);

        $mapitems = [];
        foreach ($items as $item) {
            if (isset($item['mapId']) && isset($item['objectId']) && isset($item['type']) && isset($item['uuid'])) {
                try {
                    $data = $this->_mapitem(
                        $item['objectId'],
                        $item['mapId'],
                        $item['type']
                    );

                    $mapitems[$item['uuid']] = $data;
                } catch (Exception $e) {
                    throw $e;
                }
            }

        }
        $this->set('mapitems', $mapitems);
        $this->viewBuilder()->setOption('serialize', ['mapitems']);
    }

    /**
     * @param $objectId
     * @param $mapId
     * @param $type
     * @param string $includeServiceOutput
     * @return array
     * @throws MissingDbBackendException
     */
    private function _mapitem($objectId, $mapId, $type, $includeServiceOutput = 'true') {
        if (!$objectId) {
            throw new RuntimeException('Invalid object id');
        }
        if ($mapId <= 0) {
            throw new RuntimeException('Invalid map id');
        }

        /** @var MapsTable $MapsTable */
        $MapsTable = TableRegistry::getTableLocator()->get('MapModule.Maps');

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');
        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');


        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');
        switch ($type) {
            case 'host':
                $hoststatus = $HoststatusTable->getHoststatusDetails($objectId);
                if (!empty($hoststatus)) {
                    $properties = $MapsTable->getHostInformation(
                        $ServicestatusTable,
                        $hoststatus
                    );
                }
                break;

            case 'service':
                $includeServiceOutput = $includeServiceOutput === 'true';
                $servicestatus = $ServicestatusTable->getServicestatusDetail($objectId);
                $hoststatus = $HoststatusTable->getHoststatusDetails($servicestatus->get('hostname'));

                if (!empty($servicestatus)) {
                    $properties = $MapsTable->getServiceInformation(
                        $servicestatus,
                        $hoststatus,
                        $includeServiceOutput
                    );
                    break;
                }
                break;

            case 'hostgroup':
                try {
                    /** @var HostgroupsTable $HostgroupsTable */
                    $HostgroupsTable = TableRegistry::getTableLocator()->get('Hostgroups');
                    $hostgroup = $HostgroupsTable->getHostsByHostgroupForMaps($objectId);

                    $properties = $MapsTable->getHostgroupInformation(
                        $hostgroup,
                        $HoststatusTable,
                        $ServicestatusTable
                    );
                    break;
                } catch (Exception $e) {
                }
                break;
            case 'servicegroup':
                /** @var ServicegroupsTable $ServicegroupsTable */
                $ServicegroupsTable = TableRegistry::getTableLocator()->get('Servicegroups');
                try {
                    $servicegroup = $ServicegroupsTable->getServicessByServicegroupForMaps($objectId);

                    $properties = $MapsTable->getServicegroupInformation(
                        $ServicestatusTable,
                        $servicegroup
                    );
                    break;
                } catch (RecordNotFoundException $exception) {
                }
                break;
            case 'map':
                $map = $MapsTable->getMapsForMaps($objectId, $mapId);

                if (!empty($map)) {
                    /** @var MapitemsTable $MapitemsTable */
                    $MapitemsTable = TableRegistry::getTableLocator()->get('MapModule.Mapitems');

                    //fetch all dependent map items after permissions check      5 =map on current map      8 = current map
                    $mapItemToResolve = $MapitemsTable->getMapitemsForMaps($map[0]->id, $mapId); //load sub maps on the map

                    if (!empty($mapItemToResolve)) {
                        /** @var HostgroupsTable $HostgroupsTable */
                        $HostgroupsTable = TableRegistry::getTableLocator()->get('Hostgroups');
                        /** @var ServicegroupsTable $ServicegroupsTable */
                        $ServicegroupsTable = TableRegistry::getTableLocator()->get('Servicegroups');

                        $allVisibleItems = $MapitemsTable->allVisibleMapItems($map[0]->id);
                        $mapItemIdToResolve = $mapItemToResolve['object_id'];
                        $mapIdGroupByMapId = Hash::combine(
                            $allVisibleItems,
                            '{n}.object_id',
                            '{n}.object_id',
                            '{n}.map_id'
                        );

                        if (isset($mapIdGroupByMapId[$mapItemIdToResolve])) {
                            $dependentMapsIds = $this->getDependendMaps($mapIdGroupByMapId, $mapItemIdToResolve);
                        }
                        $dependentMapsIds[] = $mapItemIdToResolve;
                        // resolve all Elements (host and/or services of dependent map)
                        $allDependentMapElements = $MapsTable->getAllDependentMapsElements(
                            $dependentMapsIds,
                            $HostgroupsTable,
                            $ServicegroupsTable
                        );

                        $hosts = [];
                        $services = [];
                        if (!empty($allDependentMapElements['hostUuids'])) {
                            $hosts = $allDependentMapElements['hostUuids'];

                            //Load all services of this host
                            $servicesByHosts = $ServicestatusTable->byHostUuids($allDependentMapElements['hostUuids']);
                            foreach ($servicesByHosts as $servicestatus) {
                                $services[$servicestatus['service_description']] = $servicestatus['service_description'];
                            }
                        }

                        if (!empty($allDependentMapElements['serviceUuids'])) {
                            //Load host uuid of this service
                            $servicestatusRecords = $ServicestatusTable->byUuids($allDependentMapElements['serviceUuids']);
                            foreach ($servicestatusRecords as $servicestatus) {
                                $hosts[$servicestatus['hostname']] = $servicestatus['hostname'];
                                $services[$servicestatus['service_description']] = $servicestatus['service_description'];
                            }
                        }

                        $properties = $MapsTable->getMapInformation(
                            $HoststatusTable,
                            $ServicestatusTable,
                            $map,
                            $hosts,
                            $services
                        );
                    }
                    break;
                }
                break;
            default:
                throw new RuntimeException('Unknown map item type');
                break;
        }

        return [
            'type' => $type,
            'data' => $properties
        ];
    }

    /**
     * @param array $maps
     * @param int $parentMapId
     * @param array &$resolvedMaps
     * @return array
     */
    public function getDependendMaps($maps, $parentMapId, &$resolvedMaps = []) {
        $allRelatedMapIdsOfParent = [];
        $resolvedMaps[$parentMapId] = $parentMapId;

        $childMapIds = $maps[$parentMapId];
        foreach ($childMapIds as $childMapId) {
            $childMapId = (int)$childMapId;
            $allRelatedMapIdsOfParent[] = $childMapId;

            if (isset($resolvedMaps[$childMapId])) {
                // We have to be inside a recursion
                // Looks a loop like 10 -> 20 -> 8 -> 10
                continue;
            }

            // Is the child map used as parent map in another relation?
            if (isset($maps[$childMapId])) {
                //Rec
                $allRelatedMapIdsOfParent = array_merge(
                    $allRelatedMapIdsOfParent,
                    $this->getDependendMaps($maps, $childMapId, $resolvedMaps)
                );
            }
        }
        return $allRelatedMapIdsOfParent;
    }

    public function mapline() {
        //Only ship template
        return;
    }

    public function mapicon() {
        //Only ship template
        return;
    }

    public function maptext() {
        //Only ship template
        return;
    }

    public function perfdatatext() {
        //Only ship template
        return;
    }

    public function serviceOutput() {
        //Only ship template
        return;
    }

    /**
     * @throws MissingDbBackendException
     */
    public function mapsummaryitem() {
        if (!$this->isApiRequest()) {
            return;
        }
        $properties = [];
        $objectId = $this->request->getQuery('objectId');
        if (!$objectId) {
            throw new RuntimeException('Invalid object id');
        }
        $mapId = (int)$this->request->getQuery('mapId');
        if ($mapId <= 0) {
            throw new RuntimeException('Invalid map id');
        }

        /** @var MapsTable $MapsTable */
        $MapsTable = TableRegistry::getTableLocator()->get('MapModule.Maps');

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');
        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        switch ($this->request->getQuery('type')) {
            case 'host':
                $host = $HoststatusTable->byUuidWithMapping($objectId);
                if (!empty($host)) {
                    $properties = $MapsTable->getHostInformationForSummaryIcon(
                        $ServicestatusTable,
                        $host
                    );
                    break;
                }
                break;

            case 'service':
                $service = $ServicestatusTable->byUuidWithMapping($objectId);

                if (!empty($service)) {
                    $properties = $MapsTable->getServiceInformationForSummaryIcon(
                        $HoststatusTable,
                        $service
                    );
                    break;
                }
                break;

            case 'hostgroup':
                /** @var HostgroupsTable $HostgroupsTable */
                $HostgroupsTable = TableRegistry::getTableLocator()->get('Hostgroups');

                $hostgroup = $HostgroupsTable->getHostsByHostgroupForMaps($objectId);

                if (!empty($hostgroup)) {
                    $properties = $MapsTable->getHostgroupInformationForSummaryIcon(
                        $HoststatusTable,
                        $ServicestatusTable,
                        $hostgroup
                    );
                    break;
                }
                break;

            case 'servicegroup':
                /** @var ServicegroupsTable $ServicegroupsTable */
                $ServicegroupsTable = TableRegistry::getTableLocator()->get('Servicegroups');

                $servicegroup = $ServicegroupsTable->getServicessByServicegroupForMaps($objectId);
                if (!empty($servicegroup)) {
                    $properties = $MapsTable->getServicegroupInformationForSummaryIcon(
                        $HoststatusTable,
                        $ServicestatusTable,
                        $servicegroup
                    );
                    break;
                }
                break;
            case 'map':
                /** @var MapsummaryitemsTable $MapsummaryitemsTable */
                $MapsummaryitemsTable = TableRegistry::getTableLocator()->get('MapModule.Mapsummaryitems');
                /** @var MapitemsTable $MapitemsTable */
                $MapitemsTable = TableRegistry::getTableLocator()->get('MapModule.Mapitems');
                /** @var HostgroupsTable $HostgroupsTable */
                $HostgroupsTable = TableRegistry::getTableLocator()->get('Hostgroups');
                /** @var ServicegroupsTable $ServicegroupsTable */
                $ServicegroupsTable = TableRegistry::getTableLocator()->get('Servicegroups');

                $map = $MapsTable->getMapsForMapsummaryitems($objectId, $mapId);
                if (!empty($map) && isset($map[0])) {
                    $map = $map[0]->toArray();
                    //fetch all dependent map items after permissions check
                    $mapSummaryItemIdToResolve = $MapsummaryitemsTable->getMapsummaryitemsForMaps($map['id'], $mapId);
                    if (!empty($mapSummaryItemIdToResolve) && isset($mapSummaryItemIdToResolve[0])) { //Chrome icon if??
                        $allVisibleItems = $MapsummaryitemsTable->allVisibleMapsummaryitems($map['id']);

                        $mapSummaryItemIdToResolve = $mapSummaryItemIdToResolve[0]->toArray()['object_id'];
                        $mapIdGroupByMapId = Hash::combine(
                            $allVisibleItems,
                            '{n}.object_id',
                            '{n}.object_id',
                            '{n}.map_id'
                        );
                        if (isset($mapIdGroupByMapId[$mapSummaryItemIdToResolve])) {
                            $dependentMapsIds = $this->getDependendMaps($mapIdGroupByMapId, $mapSummaryItemIdToResolve);
                        }
                        $dependentMapsIds[] = $mapSummaryItemIdToResolve;

                        // resolve all Elements (host and/or services of dependent map)
                        $allDependentMapElementsFromSubMaps['mapsummaryitem'] = $MapsTable->getAllDependentMapsElements(
                            $dependentMapsIds,
                            $HostgroupsTable,
                            $ServicegroupsTable
                        );
                    }

                    //fetch all dependent map items after permissions check SIMPLE STATE ITEMS (TYPE MAP)
                    $mapItemToResolve = $MapitemsTable->getMapitemsForMaps(null, $map['id']);

                    if (!empty($mapItemToResolve)) { //normal icon if???
                        $allVisibleItems = $MapitemsTable->allVisibleMapItems($map['id']);

                        $mapItemIdToResolve = $mapItemToResolve['object_id'];
                        $mapIdGroupByMapId = Hash::combine(
                            $allVisibleItems,
                            '{n}.object_id',
                            '{n}.object_id',
                            '{n}.map_id'
                        );
                        if (isset($mapIdGroupByMapId[$mapItemIdToResolve])) {
                            $dependentMapsIds = $this->getDependendMaps($mapIdGroupByMapId, $mapItemIdToResolve);
                        }
                        $dependentMapsIds[] = $mapItemIdToResolve;
                        // resolve all Elements (host and/or services of dependent map)
                        $allDependentMapElementsFromSubMaps['mapitem'] = $MapsTable->getAllDependentMapsElements(
                            $dependentMapsIds,
                            $HostgroupsTable,
                            $ServicegroupsTable
                        );
                    }

                    //simple item (host/hostgroup/service/servicegroup)
                    $allDependentMapElementsFromSubMaps['item'] = $MapsTable->getAllDependentMapsElements(
                        $map['id'],
                        $HostgroupsTable,
                        $ServicegroupsTable
                    );


                    $hosts = [];
                    $services = [];
                    if (!empty($allDependentMapElementsFromSubMaps['item']['hostUuids'])) {
                        $hosts = $allDependentMapElementsFromSubMaps['item']['hostUuids'];

                        //Load all services of this host
                        $servicesByHosts = $ServicestatusTable->byHostUuids($allDependentMapElementsFromSubMaps['item']['hostUuids']);
                        foreach ($servicesByHosts as $servicestatus) {
                            $services[$servicestatus['service_description']] = $servicestatus['service_description'];
                        }
                    }

                    if (!empty($allDependentMapElementsFromSubMaps['item']['serviceUuids'])) {
                        //Load host uuid of this service
                        $servicestatusRecords = $ServicestatusTable->byUuids($allDependentMapElementsFromSubMaps['item']['serviceUuids']);
                        foreach ($servicestatusRecords as $servicestatus) {
                            $hosts[$servicestatus['hostname']] = $servicestatus['hostname'];
                            $services[$servicestatus['service_description']] = $servicestatus['service_description'];
                        }
                    }

                    $properties = $MapsTable->getMapInformationForSummaryIcon(
                        $HoststatusTable,
                        $ServicestatusTable,
                        $map,
                        $hosts,
                        $services
                    );
                    break;
                }
                break;
            default:
                throw new RuntimeException('Unknown map item type');
                break;
        }

        $this->set('type', $this->request->getQuery('type'));
        $this->set('data', $properties);
        $this->viewBuilder()->setOption('serialize', ['type', 'data']);
    }

    public function graph() {
        if (!$this->isApiRequest()) {
            return;
        }

        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        $serviceUuid = $this->request->getQuery('serviceId', 'empty');
        $servicestatus = $ServicestatusTable->byUuidWithMapping($serviceUuid);

        if (empty($servicestatus)) {
            throw new NotFoundException('Service not found!');
        }

        $this->set('service', $servicestatus);
        $this->viewBuilder()->setOption('serialize', ['service']);
    }

    public function tacho() {
        //Only ship template
        return;
    }

    public function cylinder() {
        //Only ship template
        return;
    }

    public function trafficlight() {
        //Only ship template
        return;
    }

    public function temperature() {
        //Only ship template
        return;
    }

    public function mapsummary() {
        if (!$this->isApiRequest()) {
            return;
        }

        $objectId = $this->request->getQuery('objectId');
        $summaryStateItem = $this->request->getQuery('summary') === 'true';
        if ($this->request->getQuery('type') === 'stateless') {
            return;
        }

        if (!$objectId) {
            throw new RuntimeException('Invalid object id');
        }
        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        /** @var MapsTable $MapsTable */
        $MapsTable = TableRegistry::getTableLocator()->get('MapModule.Maps');
        /** @var HostgroupsTable $HostgroupsTable */
        $HostgroupsTable = TableRegistry::getTableLocator()->get('Hostgroups');
        /** @var ServicegroupsTable $ServicegroupsTable */
        $ServicegroupsTable = TableRegistry::getTableLocator()->get('Servicegroups');

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');
        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        switch ($this->request->getQuery('type')) {
            case 'host':

                $host = $HoststatusTable->getHoststatusDetails($objectId);

                if (!empty($host)) {
                    $summary = $MapsTable->getHostSummary(
                        $host,
                        $ServicestatusTable,
                        $UserTime
                    );
                    $this->set('type', 'host');
                    $this->set('summary', $summary);
                    $this->viewBuilder()->setOption('serialize', ['host', 'summary']);
                    return;
                }

                throw new NotFoundException('Host not found!');
                return;

                break;
            case 'service':
                $service = $ServicestatusTable->getServicestatusDetail($objectId);
                $hoststatus = $HoststatusTable->getHoststatusDetails($service->get('hostname'));

                if (!empty($service)) {
                    $summary = $MapsTable->getServiceSummary(
                        $hoststatus,
                        $service,
                        $UserTime
                    );
                    $this->set('type', 'service');
                    $this->set('summary', $summary);
                    $this->viewBuilder()->setOption('serialize', ['service', 'summary']);
                    return;
                }

                throw new NotFoundException('Service not found!');
                return;

                break;
            case 'hostgroup':
                try {
                    $hostgroup = $HostgroupsTable->getHostsByHostgroupForMaps($objectId);
                    $summary = $MapsTable->getHostgroupSummary(
                        $HoststatusTable,
                        $ServicestatusTable,
                        $hostgroup
                    );
                    $this->set('type', 'hostgroup');
                    $this->set('summary', $summary);
                    $this->viewBuilder()->setOption('serialize', ['hostgroup', 'summary']);
                    return;
                } catch (Exception $e) {
                    throw new NotFoundException('Host group not found!');
                }
                break;
            case 'servicegroup':
                $servicegroup = $ServicegroupsTable->getServicessByServicegroupForMapsWithHost($objectId);

                if (!empty($servicegroup)) {
                    $summary = $MapsTable->getServicegroupSummary(
                        $ServicestatusTable,
                        $servicegroup
                    );
                    $this->set('type', 'servicegroup');
                    $this->set('summary', $summary);
                    $this->viewBuilder()->setOption('serialize', ['servicegroup', 'summary']);
                    return;
                }

                throw new NotFoundException('Service group not found!');
                return;
                break;
            case 'map':
                /** @var MapsummaryitemsTable $MapsummaryitemsTable */
                $MapsummaryitemsTable = TableRegistry::getTableLocator()->get('MapModule.Mapsummaryitems');
                /** @var MapitemsTable $MapitemsTable */
                $MapitemsTable = TableRegistry::getTableLocator()->get('MapModule.Mapitems');

                if ($summaryStateItem) {
                    $map = $MapsTable->getMapsummaryitemForMapsummary($objectId);
                    $mapId = $map[0]['Mapsummaryitems']['map_id'];
                } else {
                    $map = $MapsTable->getMapitemForMapsummary($objectId);
                    $mapId = $map[0]['Mapitems']['map_id'];
                }

                if (!empty($map) && isset($map[0])) {
                    $map = $map[0]->toArray();
                    //fetch all dependent map items after permissions check
                    $mapSummaryItemIdToResolve = $MapsummaryitemsTable->getMapsummaryitemsForMaps($map['id'], $mapId);
                    if (!empty($mapSummaryItemIdToResolve) && isset($mapSummaryItemIdToResolve[0])) { //Chrome icon if??
                        $allVisibleItems = $MapsummaryitemsTable->allVisibleMapsummaryitems($map['id']);

                        $mapSummaryItemIdToResolve = $mapSummaryItemIdToResolve[0]->toArray()['object_id'];
                        $mapIdGroupByMapId = Hash::combine(
                            $allVisibleItems,
                            '{n}.object_id',
                            '{n}.object_id',
                            '{n}.map_id'
                        );
                        if (isset($mapIdGroupByMapId[$mapSummaryItemIdToResolve])) {
                            $dependentMapsIds = $this->getDependendMaps($mapIdGroupByMapId, $mapSummaryItemIdToResolve);
                        }
                        $dependentMapsIds[] = $mapSummaryItemIdToResolve;

                        // resolve all Elements (host and/or services of dependent map)
                        $allDependentMapElementsFromSubMaps['mapsummaryitem'] = $MapsTable->getAllDependentMapsElements(
                            $dependentMapsIds,
                            $HostgroupsTable,
                            $ServicegroupsTable
                        );
                    }

                    //fetch all dependent map items after permissions check SIMPLE STATE ITEMS (TYPE MAP)
                    $mapItemToResolve = $MapitemsTable->getMapitemsForMaps(null, $map['id']);

                    if (!empty($mapItemToResolve)) { //normal icon if???
                        $allVisibleItems = $MapitemsTable->allVisibleMapItems($map['id']);

                        $mapItemIdToResolve = $mapItemToResolve['object_id'];
                        $mapIdGroupByMapId = Hash::combine(
                            $allVisibleItems,
                            '{n}.object_id',
                            '{n}.object_id',
                            '{n}.map_id'
                        );
                        if (isset($mapIdGroupByMapId[$mapItemIdToResolve])) {
                            $dependentMapsIds = $this->getDependendMaps($mapIdGroupByMapId, $mapItemIdToResolve);
                        }
                        $dependentMapsIds[] = $mapItemIdToResolve;
                        // resolve all Elements (host and/or services of dependent map)
                        $allDependentMapElementsFromSubMaps['mapitem'] = $MapsTable->getAllDependentMapsElements(
                            $dependentMapsIds,
                            $HostgroupsTable,
                            $ServicegroupsTable
                        );
                    }

                    //simple item (host/hostgroup/service/servicegroup)
                    $allDependentMapElementsFromSubMaps['item'] = $MapsTable->getAllDependentMapsElements(
                        $map['id'],
                        $HostgroupsTable,
                        $ServicegroupsTable
                    );


                    $hosts = [];
                    $services = [];
                    if (!empty($allDependentMapElementsFromSubMaps['item']['hostUuids'])) {
                        $hosts = $allDependentMapElementsFromSubMaps['item']['hostUuids'];

                        //Load all services of this host
                        $servicesByHosts = $ServicestatusTable->byHostUuids($allDependentMapElementsFromSubMaps['item']['hostUuids']);
                        foreach ($servicesByHosts as $servicestatus) {
                            $services[$servicestatus['service_description']] = $servicestatus['service_description'];
                        }
                    }

                    if (!empty($allDependentMapElementsFromSubMaps['item']['serviceUuids'])) {
                        //Load host uuid of this service
                        $servicestatusRecords = $ServicestatusTable->byUuids($allDependentMapElementsFromSubMaps['item']['serviceUuids']);
                        foreach ($servicestatusRecords as $servicestatus) {
                            $hosts[$servicestatus['hostname']] = $servicestatus['hostname'];
                            $services[$servicestatus['service_description']] = $servicestatus['service_description'];
                        }
                    }

                    $summary = $MapsTable->getMapSummary(
                        $HoststatusTable,
                        $ServicestatusTable,
                        $map,
                        $hosts,
                        $services,
                        $UserTime,
                        $summaryStateItem
                    );

                    $this->set('type', 'map');
                    $this->set('summary', $summary);
                    $this->viewBuilder()->setOption('serialize', ['map', 'summary']);
                    return;
                }
                throw new NotFoundException('Map not found!');
                return;
                break;
            default:
                throw new RuntimeException('Unknown map item type');
                break;
        }

    }


    /**
     * @param $id
     */
    public function mapDetails($id) {
        /** @var MapsTable $MapsTable */
        $MapsTable = TableRegistry::getTableLocator()->get('MapModule.Maps');

        $id = (int)$id;
        if (!$MapsTable->existsById($id)) {
            throw new NotFoundException();
        }

        $map = $MapsTable->get($id, [])->toArray();
        $this->set('map', $map);
        $this->viewBuilder()->setOption('serialize', ['map']);
    }

    public function viewDirective() {
        //Ship template of Mapeditors view directive.
        //It is a directive be able to also use the maps as an widget
        return;
    }

    public function maps_service_browser_redirect() {
        //This method is used to lookup the hostUuid by a given servie uuid and redirect to ServicesBrowser state
        if (!$this->isApiRequest()) {
            //Only ship blank html template
            return;
        }

        if ($this->request->getQuery('uuid', null) === null) {
            throw new BadRequestException('Query Parameter &uuid= is missing!');
        }

        $serviceUuid = $this->request->getQuery('uuid');

        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        $service = $ServicestatusTable->byUuid($serviceUuid);

        if (empty($service)) {
            $this->response = $this->response->withStatus(404);
        }

        $this->set('service', $service);
        $this->viewBuilder()->setOption('serialize', ['service']);
    }
}
