<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace App\Command;

use App\Lib\SystemConfig;
use Cake\Console\Arguments;
use Cake\Command\Command;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Symfony\Component\Filesystem\Exception\IOExceptionInterface;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Rights command.
 */
class RightsCommand extends Command {
    /**
     * Hook method for defining this command's option parser.
     *
     * @see https://book.cakephp.org/3.0/en/console-and-shells/commands.html#defining-arguments-and-options
     *
     * @param \Cake\Console\ConsoleOptionParser $parser The parser to be defined
     * @return \Cake\Console\ConsoleOptionParser The built parser.
     */
    public function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser {
        $parser = parent::buildOptionParser($parser);

        return $parser;
    }

    /**
     * Implement this method with your command's logic.
     *
     * @param \Cake\Console\Arguments $args The command arguments.
     * @param \Cake\Console\ConsoleIo $io The console io
     * @return null|void|int The exit code or null for success
     */
    public function execute(Arguments $args, ConsoleIo $io) {
        $directories = [
            '/opt/openitc/frontend/'
        ];
        $this->setRights($io, $directories);
    }

    /**
     * @param ConsoleIo $io
     * @param array $dirs
     */
    private function setRights(ConsoleIo $io, $dirs = []) {
        try {

            $SystemConfig = new SystemConfig();
            $user = $SystemConfig->getWebserverUser();
            $group = $SystemConfig->getWebserverGroup();

            $fs = new Filesystem();
            foreach ($dirs as $dir) {
                if ($fs->exists($dir)) {
                    $io->out(sprintf(
                        '<info>Set filesystem permissions for user %s to %s...    </info>',
                        $user,
                        $dir
                    ), 0);
                    $fs->chown($dir, $user, true);
                    $fs->chgrp($dir, $group, true);
                    $io->out('<success>done!</success>');
                }

            }
        } catch (IOExceptionInterface $e) {
            $io->out('<error>An error occurred at ' . $e->getPath() . ' </error>');
        }
    }
}
