<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace App\Command;

use App\Model\Table\UsersTable;
use Cake\Console\Arguments;
use Cake\Command\Command;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\ORM\TableRegistry;

/**
 * User command.
 */
class UserCommand extends Command {

    /**
     * @var ConsoleIo
     */
    private $io;

    public function initialize(): void {
        \Cake\Cache\Cache::disable();
        \Cake\Cache\Cache::clear();
        parent::initialize();
    }

    /**
     * Hook method for defining this command's option parser.
     *
     * @see https://book.cakephp.org/3.0/en/console-and-shells/commands.html#defining-arguments-and-options
     *
     * @param \Cake\Console\ConsoleOptionParser $parser The parser to be defined
     * @return \Cake\Console\ConsoleOptionParser The built parser.
     */
    public function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser {
        $parser = parent::buildOptionParser($parser);

        $parser->setDescription(__('Create a new user for the Web Interface'));

        $parser->addOption('username', [
            'help'    => __('The username you want to create'),
            'default' => null
        ]);

        $parser->addOption('password', [
            'help'    => __('The password for the user you want to create'),
            'default' => null
        ]);

        $parser->addOption('usergroup-id', [
            'help'    => __('The ID of the user group'),
            'default' => 1
        ]);

        $parser->addOption('firstname', [
            'help'    => __('First name of the user'),
            'default' => 'Firstname'
        ]);

        $parser->addOption('lastname', [
            'help'    => __('Last name of the user'),
            'default' => 'Lastname'
        ]);

        return $parser;
    }

    /**
     * Implement this method with your command's logic.
     *
     * @param \Cake\Console\Arguments $args The command arguments.
     * @param \Cake\Console\ConsoleIo $io The console io
     * @return null|void|int The exit code or null for success
     */
    public function execute(Arguments $args, ConsoleIo $io) {
        $this->io = $io;

        $username = $args->getOption('username');
        $password = $args->getOption('password');
        $usergroupId = $args->getOption('usergroup-id');
        $firstname = $args->getOption('firstname');
        $lastname = $args->getOption('lastname');

        $passwordConfirm = $password;

        if (empty($username) || empty($password)) {
            $username = $this->askUsername();
            $password = $this->askPassword();
            $passwordConfirm = $this->askPasswordConfirm();
        }

        if ($password !== $passwordConfirm) {
            $this->io->error(__('Password missmatch!'));
            exit(1);
        }

        /** @var UsersTable $UsersTable */
        $UsersTable = TableRegistry::getTableLocator()->get('Users');

        $entity = $UsersTable->newEntity([
            'usergroup_id'     => $usergroupId,
            'username'         => $username,
            'firstname'        => $firstname,
            'lastname'         => $lastname,
            'password'         => $password,
            'confirm_password' => $passwordConfirm
        ]);

        $UsersTable->save($entity);

        if ($entity->hasErrors()) {
            $this->io->error(__('Error while creating user'));
            print_r($entity->getErrors());
            exit(1);
        }

        $this->io->success(__('User created successfully'));
    }

    /**
     * @return string
     */
    private function askUsername() {
        do {
            $username = trim($this->io->ask('Please enter the username.'));
        } while (empty($username));
        return $username;
    }

    /**
     * @return string
     */
    private function askPassword() {
        do {
            $password = trim($this->io->ask('Please enter your password.'));
        } while (empty($password));
        return $password;
    }

    /**
     * @return string
     */
    private function askPasswordConfirm() {
        do {
            $password = trim($this->io->ask('Please confirm your password.'));
        } while (empty($password));
        return $password;
    }
}
