<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use App\Filter\HoststatusFilter;
use App\itnovum\openITCOCKPIT\Monitoring\Naemon\ExternalCommands;
use App\Model\Entity\Hoststatus;
use App\Model\Table\DowntimeHostsTable;
use App\Model\Table\DowntimeServicesTable;
use App\Model\Table\HoststatusTable;
use App\Model\Table\ServicestatusTable;
use Cake\Http\Exception\BadRequestException;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Class ExternalcommandsController
 * @package App\Controller
 */
class ExternalcommandsController extends AppController {

    public function reset_check_time() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $type = $this->request->getQuery('type', 'host');
        $hostUuid = $this->request->getQuery('host_uuid');
        $includeServices = $this->request->getQuery('include_services', 'false') === 'true';

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');

        if (!$HoststatusTable->existsByUuid($hostUuid)) {
            throw new NotFoundException('Host not found');
        }

        if ($type === 'host') {
            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->rescheduleHost($hostUuid, $includeServices);

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        if ($type === 'service') {
            $serviceUuid = $this->request->getQuery('service_uuid');

            /** @var ServicestatusTable $ServicestatusTable */
            $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

            if (!$ServicestatusTable->existsByUuid($serviceUuid)) {
                throw new NotFoundException('Service not found');
            }

            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->rescheduleService($hostUuid, $serviceUuid);

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    public function schedule_maintenance() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $type = $this->request->getQuery('type', 'host');
        $hostUuid = $this->request->getQuery('host_uuid');
        $includeServices = $this->request->getQuery('include_services', 'false') === 'true';
        $fromDate = $this->request->getQuery('from_date', date('d.m.Y'));
        $fromTime = $this->request->getQuery('from_time', date('H:i'));
        $toDate = $this->request->getQuery('to_date', date('d.m.Y', (time() + 60 * 15)));
        $toTime = $this->request->getQuery('to_time', date('H:i', (time() + 60 * 15)));
        $comment = $this->request->getQuery('comment', __('No comment given'));

        $User = $this->getUser();
        $UserTime = UserTime::fromUser($User);
        $author = sprintf('%s %s', $User->get('lastname'), $User->get('firstname'));
        if (empty($author)) {
            $author = $User->get('username');
        }

        $startTime = sprintf('%s %s', $fromDate, $fromTime);
        $endTime = sprintf('%s %s', $toDate, $toTime);

        $startTime = strtotime($startTime);
        if ($startTime === false) {
            $startTime = time();
        }

        $endTime = strtotime($endTime);
        if ($endTime === false) {
            $endTime = time() + 15 * 60;
        }

        $offset = $UserTime->getUserTimeToServerOffset();
        $startTime = $startTime - $offset;
        $endTime = $endTime - $offset;

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');

        if (!$HoststatusTable->existsByUuid($hostUuid)) {
            throw new NotFoundException('Host not found');
        }

        if ($type === 'host') {
            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->setHostDowntime(
                $hostUuid,
                $startTime,
                $endTime,
                $author,
                $comment,
                $includeServices
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        if ($type === 'service') {
            $serviceUuid = $this->request->getQuery('service_uuid');

            /** @var ServicestatusTable $ServicestatusTable */
            $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

            if (!$ServicestatusTable->existsByUuid($serviceUuid)) {
                throw new NotFoundException('Service not found');
            }

            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->setServiceDowntime(
                $hostUuid,
                $serviceUuid,
                $startTime,
                $endTime,
                $author,
                $comment
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    public function acknowledge_status() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $type = $this->request->getQuery('type', 'host');
        $hostUuid = $this->request->getQuery('host_uuid');
        $includeServices = $this->request->getQuery('include_services', 'false') === 'true';
        $isSticky = $this->request->getQuery('is_sticky', 'false') === 'true';
        $comment = $this->request->getQuery('comment', __('No comment given'));

        $User = $this->getUser();
        $author = sprintf('%s %s', $User->get('lastname'), $User->get('firstname'));
        if (empty($author)) {
            $author = $User->get('username');
        }

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');

        if (!$HoststatusTable->existsByUuid($hostUuid)) {
            throw new NotFoundException('Host not found');
        }

        if ($type === 'host') {
            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->setHostAck(
                $hostUuid,
                $author,
                $comment,
                $isSticky,
                $includeServices
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        if ($type === 'service') {
            $serviceUuid = $this->request->getQuery('service_uuid');

            /** @var ServicestatusTable $ServicestatusTable */
            $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

            if (!$ServicestatusTable->existsByUuid($serviceUuid)) {
                throw new NotFoundException('Service not found');
            }

            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->setServiceAck(
                $hostUuid,
                $serviceUuid,
                $author,
                $comment,
                $isSticky
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    public function process_check_result() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $type = $this->request->getQuery('type', 'host');
        $hostUuid = $this->request->getQuery('host_uuid');
        $forceHardstate = $this->request->getQuery('force_hardstate', 'false') === 'true';
        $repetitions = (int)$this->request->getQuery('repetitions', 1);
        $output = $this->request->getQuery('output', __('Test alert'));
        $state = (int)$this->request->getQuery('state', 2);

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');

        if (!$HoststatusTable->existsByUuid($hostUuid)) {
            throw new NotFoundException('Host not found');
        }

        if ($type === 'host') {
            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->passiveTransferHostCheckresult(
                $hostUuid,
                $output,
                $state,
                $forceHardstate ? $repetitions : 1
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        if ($type === 'service') {
            $serviceUuid = $this->request->getQuery('service_uuid');

            /** @var ServicestatusTable $ServicestatusTable */
            $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

            if (!$ServicestatusTable->existsByUuid($serviceUuid)) {
                throw new NotFoundException('Service not found');
            }

            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->passiveTransferServiceCheckresult(
                $hostUuid,
                $serviceUuid,
                $output,
                $state,
                $forceHardstate ? $repetitions : 1
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    public function flap_detection() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $type = $this->request->getQuery('type', 'host');
        $hostUuid = $this->request->getQuery('host_uuid');
        $enable = (int)$this->request->getQuery('enabled');
        $enable = $enable === 1;

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');

        if (!$HoststatusTable->existsByUuid($hostUuid)) {
            throw new NotFoundException('Host not found');
        }

        if ($type === 'host') {
            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->enableOrDisableHostFlapdetection(
                $hostUuid,
                $enable
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        if ($type === 'service') {
            $serviceUuid = $this->request->getQuery('service_uuid');

            /** @var ServicestatusTable $ServicestatusTable */
            $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

            if (!$ServicestatusTable->existsByUuid($serviceUuid)) {
                throw new NotFoundException('Service not found');
            }

            $ExternalCommands = new ExternalCommands();
            $ExternalCommands->enableOrDisableServiceFlapdetection(
                $hostUuid,
                $serviceUuid,
                $enable
            );

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    public function cancel_downtime() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $type = $this->request->getData('type', 'host');
        $internalDowntimeId = (int)$this->request->getData('internal_downtime_id', 0);

        /** @var DowntimeHostsTable $DowntimeHostsTable */
        $DowntimeHostsTable = TableRegistry::getTableLocator()->get('DowntimeHosts');

        /** @var DowntimeServicesTable $DowntimeServicesTable */
        $DowntimeServicesTable = TableRegistry::getTableLocator()->get('DowntimeServices');

        $ExternalCommands = new ExternalCommands();

        if ($type === 'host') {
            $includeServices = $this->request->getData('include_services', 'false') === 'true' || $this->request->getData('include_services', 'false') === true;

            $downtime = $DowntimeHostsTable->getHostUuidWithDowntimeByInternalDowntimeId($internalDowntimeId);
            if (empty($downtime)) {
                $this->set('success', false);
                $this->set('message', __('Could not find downtime'));
                $this->viewBuilder()->setOption('serialize', ['success', 'message']);
                return;
            }

            $ExternalCommands->deleteHostDowntime($downtime['hostname'], $downtime);

            if ($includeServices) {
                $serviceDowntimes = $DowntimeServicesTable->getServiceDowntimesByHostUuidAndDowntime(
                    $downtime['hostname'],
                    $downtime
                );

                foreach ($serviceDowntimes as $serviceDowntime) {
                    $ExternalCommands->deleteServiceDowntime(
                        $serviceDowntime['hostname'],
                        $serviceDowntime['service_description'],
                        $serviceDowntime
                    );
                }
            }
            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        if ($type === 'service') {
            $downtime = $DowntimeServicesTable->getServiceUuidWithDowntimeByInternalDowntimeId($internalDowntimeId);
            if (empty($downtime)) {
                $this->set('success', false);
                $this->set('message', __('Could not find downtime'));
                $this->viewBuilder()->setOption('serialize', ['success', 'message']);
                return;
            }

            $ExternalCommands->deleteServiceDowntime($downtime['hostname'], $downtime['service_description'], $downtime);

            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);

    }

}

