<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use App\Filter\HoststatusFilter;
use App\Model\Entity\Hoststatus;
use App\Model\Table\AcknowledgementHostsTable;
use App\Model\Table\DowntimeHostsTable;
use App\Model\Table\HoststatusTable;
use App\Model\Table\ServicestatusTable;
use Cake\Http\Exception\BadRequestException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\Views\BBCodeParser;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Class HostsController
 * @package App\Controller
 */
class HostsController extends AppController {

    public function index() {
        if (!$this->isApiRequest()) {
            //Only ship the template for .html requests
            return;
        }

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');
        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        $HoststatusFilter = new HoststatusFilter($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $HoststatusFilter->getPage());

        $hoststatus = $HoststatusTable->getHoststatusIndex($HoststatusFilter, $PaginateOMat);

        $User = $this->getUser();
        $UserTime = UserTime::fromUser($User);
        $BBCodeParser = new BBCodeParser();

        foreach ($hoststatus as $index => $hoststatusEntity) {
            /** @var Hoststatus $hoststatusEntity */
            $hoststatusEntity->setUserTime($UserTime);
            $hoststatusEntity->setBBCodeParser($BBCodeParser);

            $serviceSummaryState = $ServicestatusTable->getSummaryStateForHoststatusIndex($hoststatusEntity->get('hostname'));
            $hoststatus[$index]['servicestatus_summary'] = $serviceSummaryState;
        }

        $this->set('hoststatus', $hoststatus);
        $this->viewBuilder()->setOption('serialize', ['hoststatus']);
    }

    public function browser() {
        if (!$this->isApiRequest()) {
            //Ship host details HTML template
            return;
        }

        if ($this->request->getQuery('uuid', null) === null) {
            throw new BadRequestException('Query Parameter &uuid= is missing!');
        }

        /** @var HoststatusTable $HoststatusTable */
        $HoststatusTable = TableRegistry::getTableLocator()->get('Hoststatus');
        $UserTime = UserTime::fromUser($this->getUser());
        $BBCodeParser = new BBCodeParser();

        $hoststatus = $HoststatusTable->getHoststatusDetails($this->request->getQuery('uuid'));
        $acknowledgement = null;
        $downtime = null;
        if ($hoststatus !== null) {
            $hoststatus->setUserTime($UserTime);
            $hoststatus->setBBCodeParser($BBCodeParser);

            if ($hoststatus->problem_has_been_acknowledged) {
                /** @var AcknowledgementHostsTable $AcknowledgementHostsTable */
                $AcknowledgementHostsTable = TableRegistry::getTableLocator()->get('AcknowledgementHosts');

                $acknowledgement = $AcknowledgementHostsTable->byUuid($hoststatus->hostname);

                if ($acknowledgement) {
                    $acknowledgement->setUserTime($UserTime);
                    $acknowledgement->setBBCodeParser($BBCodeParser);
                }
            }

            if ($hoststatus->scheduled_downtime_depth > 0) {
                /** @var DowntimeHostsTable $DowntimeHostsTable */
                $DowntimeHostsTable = TableRegistry::getTableLocator()->get('DowntimeHosts');

                $downtime = $DowntimeHostsTable->byHostUuid($hoststatus->hostname);
                $downtime->setUserTime($UserTime);
            }
        }

        $this->set('hoststatus', $hoststatus);
        $this->set('acknowledgement', $acknowledgement);
        $this->set('downtime', $downtime);
        $this->viewBuilder()->setOption('serialize', ['hoststatus', 'acknowledgement', 'downtime']);
    }

    public function icon() {
        //Ship status icon HTML template
        return;
    }

    public function menu() {

    }

    public function hoststatusdetailsmodal() {
        //Ship modal HTML template
        return;
    }

}

