<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use App\Filter\ServicestatusFilter;
use App\Model\Entity\Servicestatus;
use App\Model\Table\AcknowledgementServicesTable;
use App\Model\Table\DowntimeServicesTable;
use App\Model\Table\HoststatusTable;
use App\Model\Table\ServicestatusTable;
use Cake\Http\Exception\BadRequestException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\Views\BBCodeParser;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use Statusengine\PerfdataParser;

/**
 * Class ServicesController
 * @package App\Controller
 */
class ServicesController extends AppController {

    public function index() {
        if (!$this->isApiRequest()) {
            //Only ship the template for .html requests
            return;
        }

        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        $ServicestatusFilter = new ServicestatusFilter($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $ServicestatusFilter->getPage());

        $UserTime = UserTime::fromUser($this->getUser());
        $BBCodeParser = new BBCodeParser();

        $servicestatus = $ServicestatusTable->getServicestatusIndex($ServicestatusFilter, $PaginateOMat);

        foreach ($servicestatus as $index => $servicestatusEntity) {
            /** @var Servicestatus $servicestatusEntity */
            $servicestatusEntity->setUserTime($UserTime);
            $servicestatusEntity->setBBCodeParser($BBCodeParser);

            $servicestatusEntity->hoststatus->setUserTime($UserTime);
            $servicestatusEntity->hoststatus->setBBCodeParser($BBCodeParser);

            $servicestatus[$index] = $servicestatusEntity;
        }

        $this->set('servicestatus', $servicestatus);
        $this->viewBuilder()->setOption('serialize', ['servicestatus']);
    }

    public function browser() {
        if (!$this->isApiRequest()) {
            //Ship host details HTML template
            return;
        }

        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        if ($this->request->getQuery('serviceuuid', null) === null) {
            throw new BadRequestException('Query Parameter &serviceuuid= is missing!');
        }
        $uuid = $this->request->getQuery('serviceuuid');

        $UserTime = UserTime::fromUser($this->getUser());
        $BBCodeParser = new BBCodeParser();

        $servicestatus = $ServicestatusTable->getServicestatusDetail($uuid);

        $PerfdataParser = new PerfdataParser($servicestatus->get('perfdata'));
        $Perfdata = $PerfdataParser->parse();

        $acknowledgement = null;
        $downtime = null;
        if ($servicestatus !== null) {
            $servicestatus->setUserTime($UserTime);
            $servicestatus->setBBCodeParser($BBCodeParser);

            if ($servicestatus->problem_has_been_acknowledged) {
                /** @var AcknowledgementServicesTable $AcknowledgementServicesTable */
                $AcknowledgementServicesTable = TableRegistry::getTableLocator()->get('AcknowledgementServices');

                $acknowledgement = $AcknowledgementServicesTable->byUuid($servicestatus->service_description);
                if ($acknowledgement) {
                    $acknowledgement->setUserTime($UserTime);
                    $acknowledgement->setBBCodeParser($BBCodeParser);
                }
            }

            if ($servicestatus->scheduled_downtime_depth > 0) {
                /** @var DowntimeServicesTable $DowntimeServicesTable */
                $DowntimeServicesTable = TableRegistry::getTableLocator()->get('DowntimeServices');

                $downtime = $DowntimeServicesTable->byServiceUuid($servicestatus->service_description);
                $downtime->setUserTime($UserTime);
            }
        }

        $this->set('servicestatus', $servicestatus);
        $this->set('acknowledgement', $acknowledgement);
        $this->set('downtime', $downtime);
        $this->set('Perfdata', $Perfdata);
        $this->viewBuilder()->setOption('serialize', ['servicestatus', 'acknowledgement', 'downtime', 'Perfdata']);
    }

    public function automap() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }

        /** @var ServicestatusTable $ServicestatusTable */
        $ServicestatusTable = TableRegistry::getTableLocator()->get('Servicestatus');

        $ServicestatusFilter = new ServicestatusFilter($this->request);
        $servicestatus = $ServicestatusTable->getServicesForAutomap($ServicestatusFilter);

        $services = [];
        foreach ($servicestatus as $record) {
            if (!isset($services[$record['hostname']])) {
                $services[$record['hostname']] = [
                    'hoststatus'   => [
                        'current_state'                 => $record['Hoststatus']['current_state'],
                        'is_flapping'                   => $record['Hoststatus']['is_flapping'] === '1',
                        'problem_has_been_acknowledged' => $record['Hoststatus']['problem_has_been_acknowledged'],
                        'scheduled_downtime_depth'      => $record['Hoststatus']['scheduled_downtime_depth'],
                        'flap_detection_enabled'        => $record['Hoststatus']['flap_detection_enabled']
                    ],
                    'display_name' => $record['HostMapping']['display_name'],
                    'hostname'     => $record['hostname']
                ];
                $services[$record['hostname']]['servicestatus'] = [];
            }
            $services[$record['hostname']]['servicestatus'][] = $record;
        }

        $this->set('services', $services);
        $this->viewBuilder()->setOption('serialize', ['services']);

    }

    public function icon() {
        //Ship service status icon HTML template
        return;
    }

    public function menu() {
        //Ship service menu HTML template
        return;
    }


    public function servicestatusdetailsmodal() {
        //Ship modal HTML template
        return;
    }

}

