<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Model\Entity;

use Cake\ORM\Entity;
use itnovum\openITCOCKPIT\Core\Views\BBCodeParser;
use itnovum\openITCOCKPIT\Core\Views\UserTime;

/**
 * AcknowledgementHost Entity
 *
 * @property string|null $hostname
 * @property int|null $state
 * @property string|null $author_name
 * @property string|null $comment_data
 * @property int $entry_time
 * @property int|null $acknowledgement_type
 * @property bool|null $is_sticky
 * @property bool|null $persistent_comment
 * @property bool|null $notify_contacts
 */
class AcknowledgementHost extends Entity {

    /**
     * @var UserTime|null
     */
    private $UserTime = null;

    /**
     * @var BBCodeParser|null
     */
    private $BBCodeParser = null;

    /**
     * Fields that can be mass assigned using newEntity() or patchEntity().
     *
     * Note that when '*' is set to true, this allows all unspecified fields to
     * be mass assigned. For security purposes, it is advised to set '*' to false
     * (or remove it), and explicitly make individual fields accessible as needed.
     *
     * @var array
     */
    protected $_accessible = [
        'hostname'             => true,
        'state'                => true,
        'author_name'          => true,
        'comment_data'         => true,
        'entry_time'           => true,
        'acknowledgement_type' => true,
        'is_sticky'            => true,
        'persistent_comment'   => true,
        'notify_contacts'      => true,
    ];

    /**
     * @param UserTime $UserTime
     */
    public function setUserTime(UserTime $UserTime) {
        $this->UserTime = $UserTime;
    }

    public function setBBCodeParser(BBCodeParser $BBCodeParser) {
        $this->BBCodeParser = $BBCodeParser;
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function jsonSerialize(): array {
        if ($this->UserTime === null && $this->BBCodeParser === null) {
            return parent::jsonSerialize();
        }

        $data = parent::jsonSerialize();
        if ($this->UserTime) {
            $fields = [
                'entry_time'
            ];

            foreach ($fields as $field) {
                if (isset($data[$field])) {
                    $data[$field] = $this->UserTime->format($data[$field]);
                }
            }
        }

        if ($this->BBCodeParser) {
            $bbFields = [
                'comment_data'
            ];

            foreach ($bbFields as $field) {
                if (isset($data[$field])) {
                    $data[$field . '_html'] = $this->BBCodeParser->nagiosNl2br($this->BBCodeParser->asHtml($this->{$field}, true));
                }
            }
        }

        return $data;
    }
}
