<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Model\Table;

use App\Filter\HostcheckFilter;
use App\Lib\Traits\PaginationAndScrollIndexTrait;
use App\Model\Entity\Hostcheck;
use Cake\Datasource\EntityInterface;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use itnovum\openITCOCKPIT\Core\HostcheckConditions;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Hostchecks Model
 *
 * @method Hostcheck get($primaryKey, $options = [])
 * @method Hostcheck newEntity($data = null, array $options = [])
 * @method Hostcheck[] newEntities(array $data, array $options = [])
 * @method Hostcheck|bool save(EntityInterface $entity, $options = [])
 * @method Hostcheck patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method Hostcheck[] patchEntities($entities, array $data, array $options = [])
 * @method Hostcheck findOrCreate($search, callable $callback = null, $options = [])
 */
class HostchecksTable extends Table {
    use PaginationAndScrollIndexTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('statusengine_hostchecks');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        //Readonly table
        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        //Readonly table
        return $rules;
    }

    /**
     * @param HostcheckConditions $HostcheckConditions
     * @param PaginateOMat|null $PaginateOMat
     * @return array
     */
    public function getHostchecks(HostcheckConditions $HostcheckConditions, $PaginateOMat = null) {

        $query = $this->find()
            ->where([
                'Hostchecks.start_time >' => $HostcheckConditions->getFrom(),
                'Hostchecks.start_time <' => $HostcheckConditions->getTo()
            ])
            ->order($HostcheckConditions->getOrder());

        if ($HostcheckConditions->hasConditions()) {
            $query->andWhere($HostcheckConditions->getConditions());
        }

        if (!empty($HostcheckConditions->getStates())) {
            $query->andWhere([
                'Hostchecks.state IN' => $HostcheckConditions->getStates()
            ]);
        }

        if (!empty($HostcheckConditions->getStateTypes())) {
            $query->andWhere([
                'Hostchecks.is_hardstate IN' => $HostcheckConditions->getStateTypes()
            ]);
        }

        if ($PaginateOMat === null) {
            //Just execute query
            $result = $this->emptyArrayIfNull($query->toArray());
        } else {
            if ($PaginateOMat->useScroll()) {
                $result = $this->scrollCake4($query, $PaginateOMat->getHandler());
            } else {
                $result = $this->paginateCake4($query, $PaginateOMat->getHandler());
            }
        }

        return $result;
    }
}
