<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace itnovum\openITCOCKPIT\Core\AngularJS;

use Cake\Filesystem\Folder;

class AngularAssets implements AngularAssetsInterface {

    /**
     * @var array
     */
    protected $jsFiles = [
        'node_modules/jquery/dist/jquery.min.js',
        'node_modules/jquery-ui-dist/jquery-ui.min.js',
        'node_modules/popper.js/dist/umd/popper.min.js',
        'node_modules/popper.js/dist/umd/popper-utils.min.js',
        'node_modules/angular/angular.min.js',
        'node_modules/bootstrap/dist/js/bootstrap.bundle.min.js',
        'node_modules/angular-ui-router/release/angular-ui-router.min.js',
        'js/jquery-snippets.js',
        'js/lib/php.js',
        'js/lib/colr.js',
        'js/lib/throttle-debounce/jquery.ba-throttle-debounce.min.js',
        'node_modules/noty/lib/noty.js',
        'node_modules/tsparticles/tsparticles.min.js',
        'js/lib/parseuri.js',
        'js/lib/chosen.jquery.min.js',
        'node_modules/dropzone/dist/min/dropzone.min.js',
        'js/lib/jquery.svg.min.js',
        'js/lib/jquery.svgfilter.min.js',
        'js/lib/jquery.flot.cust.js', //legacy but no new alternative
        'node_modules/flot-axislabels/jquery.flot.axislabels.js',
        'node_modules/flot/jquery.flot.time.js',
        'node_modules/flot/jquery.flot.fillbetween.js',
        'node_modules/flot/jquery.flot.resize.js',
        'node_modules/flot/jquery.flot.threshold.js',
        'node_modules/flot/jquery.flot.selection.js',
        'js/lib/jquery.flot.curvedLines.js',
        'js/lib/GraphDefaults.js',
        'js/lib/uPlotGraphDefaults.js',
        'node_modules/canvas-gauges/gauge.min.js',
        'node_modules/luxon/build/global/luxon.min.js',
        'node_modules/uplot/dist/uPlot.iife.min.js',
        'node_modules/jsoneditor/dist/jsoneditor.min.js'
    ];

    /**
     * @var array
     * Gets added before self::cssFiles in the HTML document
     */
    protected $nodeCssFiles = [
        '/node_modules/jquery-ui-dist/jquery-ui.css',
        '/node_modules/font-awesome/css/font-awesome.css',
        '/node_modules/@fortawesome/fontawesome-free/css/all.min.css',
        '/node_modules/dropzone/dist/min/dropzone.min.css',
        '/node_modules/noty/lib/noty.css',
        '/node_modules/bootstrap4c-chosen/dist/css/component-chosen.css',
        '/node_modules/dropzone/dist/min/dropzone.min.css',
        '/node_modules/animate.css/animate.min.css',
        '/node_modules/uplot/dist/uPlot.min.css',
        '/node_modules/jsoneditor/dist/jsoneditor.min.css'
    ];

    /**
     * @var array
     */
    protected $cssFiles = [
        '/css/roboto.css',
        '/smartadmin4/dist/css/vendors.bundle.css',
        '/smartadmin4/dist/css/app.bundle.css',
        '/smartadmin4/dist/css/themes/cust-theme-10.css',
        '/css/lib/jquery-jvectormap-1.2.2.css',
        '/css/app.css',
        '/css/openitcockpit-colors.css',
        '/css/openitcockpit-utils.css',
        '/css/openitcockpit.css',
        '/css/openitcockpit-menu.css',
        '/css/openitcockpit-vis.css'
    ];

    /**
     * @return array
     */
    public function getJsFiles() {
        $jsFiles = $this->jsFiles;
        //Load Plugin configuration files

        $Folder = new Folder(PLUGIN);
        $folders = $Folder->subdirectories();

        $loadedModules = array_filter($folders, function ($value) {
            return strpos($value, 'Module') !== false;
        });

        foreach ($loadedModules as $loadedModule) {
            $file = $loadedModule . DS . 'src' . DS . 'Lib' . DS . 'AngularAssets.php';
            if (file_exists($file)) {
                require_once $file;
                $moduleNameArray = explode('/', $loadedModule);
                $moduleName = array_pop($moduleNameArray);
                $dynamicAngularAssets = sprintf('itnovum\openITCOCKPIT\%s\AngularAssets\AngularAssets', $moduleName);
                $ModuleAngularAssets = new $dynamicAngularAssets();
                /** @var AngularAssetsInterface $ModuleAngularAssets */
                foreach ($ModuleAngularAssets->getJsFiles() as $jsFile) {
                    $jsFiles[] = $jsFile;
                }
            }
        }
        return $jsFiles;
    }

    /**
     * @return array
     */
    public function getCssFiles() {
        $cssFiles = $this->cssFiles;
        //Load Plugin configuration files

        $Folder = new Folder(PLUGIN);
        $folders = $Folder->subdirectories();

        $loadedModules = array_filter($folders, function ($value) {
            return strpos($value, 'Module') !== false;
        });

        foreach ($loadedModules as $loadedModule) {
            $file = $loadedModule . DS . 'src' . DS . 'Lib' . DS . 'AngularAssets.php';
            if (file_exists($file)) {
                require_once $file;
                $moduleNameArray = explode('/', $loadedModule);
                $moduleName = array_pop($moduleNameArray);
                $dynamicAngularAssets = sprintf('itnovum\openITCOCKPIT\%s\AngularAssets\AngularAssets', $moduleName);
                $ModuleAngularAssets = new $dynamicAngularAssets();
                /** @var AngularAssetsInterface $ModuleAngularAssets */
                foreach ($ModuleAngularAssets->getCssFiles() as $cssFile) {
                    $cssFiles[] = $cssFile;
                }
            }
        }
        return $cssFiles;
    }

    public function getNodeCssFiles() {
        return $this->nodeCssFiles;
    }

    /**
     * @return array
     */
    public function getCssFilesOnDisk() {
        $cssFiles = [];
        foreach ($this->cssFiles as $cssFile) {
            if (substr($cssFile, 0, 1) === '/') {
                //Remove leading / from path
                $cssFile = substr($cssFile, 1);
            }

            $cssFiles[] = WWW_ROOT . $cssFile;
        }

        //Load Plugin configuration files
        $Folder = new Folder(PLUGIN);
        $folders = $Folder->subdirectories();

        $loadedModules = array_filter($folders, function ($value) {
            return strpos($value, 'Module') !== false;
        });

        foreach ($loadedModules as $loadedModule) {
            $file = $loadedModule . DS . 'src' . DS . 'Lib' . DS . 'AngularAssets.php';
            if (file_exists($file)) {
                require_once $file;
                $moduleNameArray = explode('/', $loadedModule);
                $moduleName = array_pop($moduleNameArray);
                $dynamicAngularAssets = sprintf('itnovum\openITCOCKPIT\%s\AngularAssets\AngularAssets', $moduleName);
                $ModuleAngularAssets = new $dynamicAngularAssets();
                /** @var AngularAssetsInterface $ModuleAngularAssets */
                foreach ($ModuleAngularAssets->getCssFilesOnDisk() as $cssFile) {
                    $cssFiles[] = $cssFile;
                }
            }
        }
        return $cssFiles;
    }

    /**
     * @return array
     */
    public function getNodeCssFilesOnDisk() {
        $cssFiles = [];
        foreach ($this->nodeCssFiles as $cssFile) {
            if (substr($cssFile, 0, 1) === '/') {
                //Remove leading / from path
                $cssFile = substr($cssFile, 1);
            }

            $cssFiles[] = WWW_ROOT . $cssFile;
        }
        return $cssFiles;
    }

    /**
     * @return array
     */
    public function getNodeJsFiles() {
        $jsFiles = [];
        foreach ($this->jsFiles as $jsFile) {
            if (substr($jsFile, 0, 1) === '/') {
                //Remove leading / from path
                $jsFile = substr($jsFile, 1);
            }

            if (substr($jsFile, 0, 12) === 'node_modules') {
                $jsFiles[] = $jsFile;
            }
        }
        return $jsFiles;
    }

    /**
     * @return array
     */
    public function getJsFilesOnDisk() {
        // TODO: Implement getJsFilesOnDisk() method.
    }

    public function getPluginNgStateJsFiles() {
        $ngStateJsFiles = [];

        $Folder = new Folder(PLUGIN);
        $folders = $Folder->subdirectories();
        $loadedModules = array_filter($folders, function ($value) {
            return strpos($value, 'Module') !== false;
        });

        foreach ($loadedModules as $loadedModule) {
            //Also include ng.stats.js of the Plugin
            $ngStateJs = $loadedModule . DS . 'webroot' . DS . 'js' . DS . 'scripts' . DS . 'ng.states.js';
            if (file_exists($ngStateJs)) {
                $jsFiles[] = $ngStateJs;
            }
        }

        if (isset($jsFiles)) {
            return $jsFiles;
        } else {
            return [];
        }

    }
}
