<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace App\itnovum\openITCOCKPIT\Core\Permissions;


use Acl\Model\Table\AcosTable;
use Acl\Model\Table\ArosTable;
use App\Model\Table\UsersTable;
use Cake\ORM\Query;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use Cake\Utility\Inflector;

class MyRightsFactory {
    /**
     * @param $userId
     * @param $userGroupId
     * @return array
     */
    public static function getUserPermissions($userId, $usergroupId) {
        /** @var UsersTable $UsersTable */
        $UsersTable = TableRegistry::getTableLocator()->get('Users');

        $user = $UsersTable->getUserById($userId);

        /** @var ArosTable $ArosTable */
        $ArosTable = TableRegistry::getTableLocator()->get('Acl.Aros');
        /** @var AcosTable $AcosTable */
        $AcosTable = TableRegistry::getTableLocator()->get('Acl.Acos');

        $AcosAros = $ArosTable->find()
            ->where([
                'Aros.foreign_key' => $usergroupId
            ])
            ->contain([
                'Acos' => function (Query $query) {
                    return $query->where([
                        '_create' => '1'
                    ]);
                }
            ])
            ->disableHydration()
            ->first();

        $acos = $AcosAros['acos'];

        $acoIdsOfUsergroup = Hash::combine($acos, '{n}.id', '{n}.id');
        unset($acos, $AcosAros);

        $acos = $AcosTable->find('threaded')
            ->disableHydration()
            ->all();

        $acos = $acos->toArray();

        $permissions = [];
        foreach ($acos as $usergroupAcos) {
            foreach ($usergroupAcos['children'] as $controllerAcos) {
                $controllerName = strtolower($controllerAcos['alias']);
                if (!strpos($controllerName, 'module')) {
                    //Core
                    foreach ($controllerAcos['children'] as $actionAcos) {
                        //Check if the user group is allowd for $actionAcos action
                        if (!isset($acoIdsOfUsergroup[$actionAcos['id']])) {
                            continue;
                        }
                        $actionName = strtolower($actionAcos['alias']);
                        $permissions[$controllerName][$actionName] = $actionName;
                    }
                } else {
                    //Plugin / Module
                    $pluginName = Inflector::underscore($controllerName);
                    $pluginAcos = $controllerAcos;
                    foreach ($pluginAcos['children'] as $controllerAcos) {

                        $controllerName = strtolower($controllerAcos['alias']);
                        foreach ($controllerAcos['children'] as $actionAcos) {
                            //Check if the user group is allowd for $actionAcos action
                            if (!isset($acoIdsOfUsergroup[$actionAcos['id']])) {
                                continue;
                            }
                            $actionName = strtolower($actionAcos['alias']);
                            $permissions[$pluginName][$controllerName][$actionName] = $actionName;
                        }
                    }
                }
            }
        }

        $userPermissions = [
            'PERMISSIONS' => $permissions
        ];

        return $userPermissions;
    }
}
