<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace itnovum\openITCOCKPIT\Core\ValueObjects;


use itnovum\openITCOCKPIT\Exceptions\UnknownStateException;

class HostStates {

    /**
     * @var array
     */
    private $stateNames = [
        0 => 'recovery',
        1 => 'down',
        2 => 'unreachable'
    ];

    /**
     * @var array
     */
    private $stateIds = [
        'recovery'    => 0,
        'down'        => 1,
        'unreachable' => 2
    ];

    /**
     * @var array
     */
    private $states = [
        'recovery'    => false,
        'down'        => false,
        'unreachable' => false
    ];

    /**
     * @param int $state
     * @param bool $value
     */
    public function setState($state, $value) {
        $key = $this->getStateNameByState($state);
        $this->states[$key] = $value;
    }

    /**
     * @return bool
     */
    public function hasRecovery() {
        return $this->states['recovery'];
    }

    /**
     * @return bool
     */
    public function hasDown() {
        return $this->states['down'];
    }

    /**
     * @return bool
     */
    public function hasUnreachable() {
        return $this->states['unreachable'];
    }

    /**
     * @return array
     */
    public function asArray() {
        return $this->states;
    }


    /**
     * @return array
     */
    public function asIntegerArray() {
        $states = [];
        if ($this->hasRecovery()) {
            $states[] = 0;
        }
        if ($this->hasDown()) {
            $states[] = 1;
        }
        if ($this->hasUnreachable()) {
            $states[] = 2;
        }
        return $states;
    }

    /**
     * @return array
     */
    public function getAvailableStateIds() {
        return $this->stateIds;
    }

    /**
     * @param int $state
     * @return string
     * @throws UnknownStateException
     */
    public function getStateNameByState($state) {
        if (isset($this->stateNames[$state])) {
            return $this->stateNames[$state];
        }

        throw new UnknownStateException(sprintf('State "%s" is not a valid state id', $state));
    }

}
