<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace itnovum\openITCOCKPIT\Database;

use Cake\Controller\Controller;

class ScrollIndex {

    /**
     * Current (CakePHP 4) Controller
     * @var Controller
     */
    private $Controller;

    /**
     * @var int
     */
    private $limit = 25;

    /**
     * @var int
     */
    private $page = 1;

    /**
     * @var bool
     */
    private $hasNextPage = true;


    /**
     * ScrollIndex constructor.
     * @param Controller $Controller
     * @param int $page
     * @param int $limit
     */
    public function __construct(Controller $Controller, $page = 1, $limit = 25) {
        $this->Controller = $Controller;

        $this->page = (int)$page;
        $this->limit = (int)$limit;

        if ($this->page === 0 || $this->page < 0) {
            $this->page = 1;
        }

        if ($this->limit === 0 || $this->limit < 0) {
            $this->limit = 25;
        }
    }

    public function determineHasNextPage($dbResults) {
        if (is_array($dbResults)) {
            $this->hasNextPage = sizeof($dbResults) === $this->limit;
            return;
        }
        $this->hasNextPage = true;
    }

    /**
     * @param int $page
     */
    public function setPage($page) {
        $this->page = $page;
    }

    /**
     * @return int
     */
    public function getLimit() {
        return $this->limit;
    }

    public function scroll() {
        $this->Controller->set('scroll', $this->getScroll());
    }

    public function getScroll() {
        return [
            'page'        => $this->page,
            'limit'       => $this->limit,
            'offset'      => $this->getOffset(),
            'hasPrevPage' => $this->page !== 1,
            'prevPage'    => ($this->page !== 1) ? $this->page - 1 : 1,
            'nextPage'    => $this->page + 1,
            'current'     => $this->page,
            'hasNextPage' => $this->hasNextPage
        ];
    }

    /**
     * @return int
     */
    public function getOffset() {
        //return (int)$this->limit * $this->page;
        if ($this->page === 1) {
            return 0;
        }
        return (int)$this->limit * ($this->page - 1);
    }

}
