angular.module('openITCOCKPIT')
    .controller('DowntimesHostController', function($scope, $http, $rootScope, $httpParamSerializer, SortService, QueryStringService, MassChangeService, $interval, $stateParams, $q){

        SortService.setSort(QueryStringService.getValue('sort', 'DowntimeHosts.scheduled_start_time'));
        SortService.setDirection(QueryStringService.getValue('direction', 'desc'));
        $scope.currentPage = 1;


        var now = new Date();

        $scope.useScroll = true;

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                DowntimeHosts: {
                    author_name: '',
                    comment_data: '',
                    was_cancelled: false,
                    was_not_cancelled: false
                },
                from: date('d.m.Y H:i', now.getTime() / 1000 - (3600 * 24 * 30)),
                to: date('d.m.Y H:i', now.getTime() / 1000 + (3600 * 24 * 30 * 2)),
                isRunning: false,
                hideExpired: true
            };
        };
        /*** Filter end ***/

        $scope.massChange = {};
        $scope.selectedElements = 0;

        $scope.init = true;
        $scope.showFilter = false;

        $scope.showFlashMsg = function(){
            new Noty({
                theme: 'metroui',
                type: 'success',
                layout: 'topCenter',
                text: $scope.flashMshStr,
                timeout: 4000
            }).show();

            $scope.showFlashSuccess = true;
            $scope.autoRefreshCounter = 5;
            var interval = $interval(function(){
                $scope.autoRefreshCounter--;
                if($scope.autoRefreshCounter === 0){
                    $scope.load();
                    $interval.cancel(interval);
                    $scope.showFlashSuccess = false;
                }
            }, 1000);
        };

        $scope.load = function(){
            var wasCancelled = '';
            if($scope.filter.DowntimeHosts.was_cancelled ^ $scope.filter.DowntimeHosts.was_not_cancelled){
                wasCancelled = $scope.filter.DowntimeHosts.was_cancelled === true;
            }
            $q.all([
                $http.get("/downtimes/host.json", {
                    params: {
                        'angular': true,
                        'scroll': $scope.useScroll,
                        'sort': SortService.getSort(),
                        'page': $scope.currentPage,
                        'direction': SortService.getDirection(),
                        'filter[DowntimeHosts.hostname]': $stateParams.uuid,
                        'filter[DowntimeHosts.author_name]': $scope.filter.DowntimeHosts.author_name,
                        'filter[DowntimeHosts.comment_data]': $scope.filter.DowntimeHosts.comment_data,
                        'filter[DowntimeHosts.was_cancelled]': wasCancelled,
                        'filter[from]': $scope.filter.from,
                        'filter[to]': $scope.filter.to,
                        'filter[hideExpired]': $scope.filter.hideExpired,
                        'filter[isRunning]': $scope.filter.isRunning
                    }
                }),
                $http.get("/hosts/browser.json", {
                    params: {
                        uuid: $stateParams.uuid
                    }
                })
            ]).then(function(results){
                $scope.downtimes = results[0].data.all_host_downtimes;
                $scope.paging = results[0].data.paging;
                $scope.scroll = results[0].data.scroll;

                $scope.hoststatus = results[1].data.hoststatus;
                $scope.init = false;
            });
        };


        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
        };

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };


        $scope.selectAll = function(){
            if($scope.downtimes){
                for(var key in $scope.downtimes){
                    if($scope.downtimes[key].is_cancellable){
                        var id = $scope.downtimes[key].internal_downtime_id;
                        $scope.massChange[id] = true;
                    }
                }
            }
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.getObjectForDelete = function(downtime){
            var object = {};
            object[downtime.internal_downtime_id] = {
                Host: {
                    uuid: downtime.hostname
                },
                Downtime: {
                    internal_downtime_id: downtime.internal_downtime_id
                }
            };
            return object;
        };

        $scope.getObjectsForDelete = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.downtimes){
                for(var id in selectedObjects){
                    if($scope.downtimes[key].is_cancellable){
                        if(id == $scope.downtimes[key].internal_downtime_id){
                            objects[id] = {
                                Host: {
                                    uuid: $scope.downtimes[key].hostname
                                },
                                Downtime: {
                                    internal_downtime_id: $scope.downtimes[key].internal_downtime_id
                                }
                            };
                        }
                    }
                }
            }
            return objects;
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };


        //Fire on page load
        defaultFilter();
        SortService.setCallback($scope.load);

        $scope.$watch('filter', function(){
            $scope.currentPage = 1;
            $scope.load();
        }, true);

        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);

    });
