angular.module('openITCOCKPIT')
    .controller('HostsIndexController', function($scope, $http, $stateParams, $interval, SortService, MassChangeService){

        $scope.currentPage = 1;
        $scope.useScroll = true;

        SortService.setSort('current_state');
        SortService.setDirection('desc');

        /** Filter settings **/
        var defaultFilter = function(){
            $scope.stateUp = true;
            $scope.stateDown = true;
            $scope.stateUnreachable = true;
            $scope.current_state = [];
            $scope.filter_host_name = $stateParams.hostname || '';
            $scope.filter_output = '';
            $scope.showFilter = false;


            $scope.filter = {
                Hoststatus: {
                    acknowledged: false,
                    not_acknowledged: false,
                    in_downtime: false,
                    not_in_downtime: false
                }
            };
        };

        defaultFilter();
        /** End filter Settings **/

        $scope.massChange = {};
        $scope.init = true;

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
        };

        $scope.showFlashMsg = function(){
            new Noty({
                theme: 'metroui',
                type: 'success',
                layout: 'topCenter',
                text: $scope.flashMshStr,
                timeout: 4000
            }).show();

            $scope.showFlashSuccess = true;
            $scope.autoRefreshCounter = 5;
            var interval = $interval(function(){
                $scope.autoRefreshCounter--;
                if($scope.autoRefreshCounter === 0){
                    $scope.load();
                    $interval.cancel(interval);
                    $scope.showFlashSuccess = false;
                }
            }, 1000);
        };

        $scope.load = function(){
            var hasBeenAcknowledged = '';
            var inDowntime = '';
            if($scope.filter.Hoststatus.acknowledged ^ $scope.filter.Hoststatus.not_acknowledged){
                hasBeenAcknowledged = $scope.filter.Hoststatus.acknowledged === true;
            }
            if($scope.filter.Hoststatus.in_downtime ^ $scope.filter.Hoststatus.not_in_downtime){
                inDowntime = $scope.filter.Hoststatus.in_downtime === true;
            }

            $http.get("/hosts/index.json", {
                params: {
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'filter[current_state][]': $scope.current_state,
                    'filter[HostMapping.display_name]': $scope.filter_host_name,
                    'filter[output]': $scope.filter_output,
                    'filter[Hoststatus.problem_has_been_acknowledged]': hasBeenAcknowledged,
                    'filter[Hoststatus.scheduled_downtime_depth]': inDowntime
                }
            }).then(function(result){
                $scope.hoststatus = result.data.hoststatus;
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;
                $scope.init = false;
            });

        };

        $scope.parseUrlFilter = function(){
            if($stateParams.statename){
                $scope.stateUp = false;
                $scope.stateDown = false;
                $scope.stateUnreachable = false;
                switch($stateParams.statename){
                    case 'up':
                        $scope.stateUp = true;
                        break;
                    case 'down':
                        $scope.stateDown = true;
                        break;
                    default:
                        $stateParams.statename = 'unreachable';
                        $scope.stateUnreachable = true;
                        break;
                }
                $scope.current_state = [$stateParams.statename];
            }
        };

        $scope.changepage = function(page){
            $scope.undoSelection();
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        $scope.selectAll = function(){
            if($scope.hoststatus){
                for(var key in $scope.hoststatus){
                    var uuid = $scope.hoststatus[key].hostname;
                    $scope.massChange[uuid] = true;
                }
            }
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.getObjectsForExternalCommand = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.hoststatus){
                for(var uuid in selectedObjects){
                    if(uuid === $scope.hoststatus[key].hostname){
                        objects[uuid] = {
                            Host: {
                                uuid: $scope.hoststatus[key].hostname
                            }
                        }
                    }

                }
            }
            //console.log(objects);
            return objects;
        };


        // Fire on page load
        $scope.$watchGroup(['stateUp', 'stateDown', 'stateUnreachable', 'filter_host_name', 'filter_output'], function(){
            if($scope.init){
                return;
            }
            $scope.current_state = [];
            $scope.undoSelection();
            if($scope.stateUp){
                $scope.current_state.push('up');
            }
            if($scope.stateDown){
                $scope.current_state.push('down');
            }
            if($scope.stateUnreachable){
                $scope.current_state.push('unreachable');
            }
            $scope.load();
        });

        $scope.$watch('filter', function(){
            if($scope.init){
                return;
            }

            $scope.load();
        }, true);

        $scope.parseUrlFilter();
        SortService.setCallback($scope.load);
        SortService.triggerReload();


        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);

    });
