angular.module('openITCOCKPIT')
    .controller('ServicesBrowserController', function($scope, $interval, $http, $stateParams, $q, LocalStorageService){

        $scope.init = true;
        $scope.selectedTab = 'tab1';

        /*** Graph variables ***/
        $scope.isLoadingGraph = false;
        $scope.dataSources = [];
        $scope.currentDataSource = null;
        $scope.serverTimeDateObject = null;
        $scope.availableTimeranges = {
            1: '1 hour',
            2: '2 hours',
            3: '3 hours',
            4: '4 hours',
            8: '8 hours',
            24: '1 day',
            48: '2 days',
            120: '5 days',
            168: '7 days',
            720: '30 days',
            2160: '90 days',
            4464: '6 months',
            8760: '1 year'
        };
        $scope.currentSelectedTimerange = 3;
        $scope.currentAggregation = 'avg';
        $scope.graph = {
            graphAutoRefresh: true,
            showDatapoints: false,
            smoothInterpolation: LocalStorageService.getItemWithDefault('smoothGraphInterpolation', 'false') === 'true',
        };
        $scope.graphAutoRefreshInterval = 0;
        $scope.currentGraphUnit = null;
        var zoomCallbackWasBind = false;
        var graphAutoRefreshIntervalId = null;
        var lastGraphStart = 0;
        var lastGraphEnd = 0;
        var graphRenderEnd = 0;
        /*** Graph end ***/

        /*** "Private" Methods ***/
        var loadGraph = function(hostUuid, serviceuuid, appendData, start, end, saveStartAndEnd){
            if(saveStartAndEnd){
                lastGraphStart = start;
                lastGraphEnd = end;
            }

            //The last timestamp in the y-axe
            graphRenderEnd = end;

            if($scope.dataSources.length > 0){
                $scope.isLoadingGraph = true;

                var params = {
                    angular: true,
                    host_uuid: hostUuid,
                    service_uuid: serviceuuid,
                    start: start,
                    end: end,
                    jsTimestamp: 1,
                    gauge: $scope.currentDataSource,
                    aggregation: $scope.currentAggregation
                };

                if($scope.currentGraphUnit !== null){
                    params.forcedUnit = $scope.currentGraphUnit;
                }

                $http.get('/Graphgenerators/getPerfdataByUuid.json', {
                    params: params
                }).then(function(result){
                    $scope.isLoadingGraph = false;
                    if(appendData === false){
                        //Did we got date from Server?
                        if(result.data.performance_data.length > 0){
                            //Use the first metrics the server gave us.
                            $scope.perfdata = {
                                datasource: result.data.performance_data[0].datasource,
                                data: {}
                            };
                            //Convert Servertime into user time
                            for(var timestamp in result.data.performance_data[0].data){
                                var frontEndTimestamp = (parseInt(timestamp, 10));
                                $scope.perfdata.data[frontEndTimestamp] = result.data.performance_data[0].data[timestamp];
                            }
                        }else{
                            $scope.perfdata = {
                                data: {},
                                datasource: {
                                    ds: null,
                                    name: null,
                                    label: null,
                                    unit: null,
                                    act: null,
                                    warn: null,
                                    crit: null,
                                    min: null,
                                    max: null
                                }
                            };
                        }
                    }

                    if(appendData === true){
                        if(result.data.performance_data.length > 0){
                            //Append new data to current graph
                            for(var timestamp in result.data.performance_data[0].data){
                                var frontEndTimestamp = (parseInt(timestamp, 10));
                                $scope.perfdata.data[frontEndTimestamp] = result.data.performance_data[0].data[timestamp];
                            }
                        }
                    }
                    if($scope.graph.graphAutoRefresh === true && $scope.graphAutoRefreshInterval > 1000){
                        enableGraphAutorefresh();
                    }

                    //Save current unit for auto refresh
                    $scope.currentGraphUnit = $scope.perfdata.datasource.unit;
                    renderGraph($scope.perfdata);
                });
            }
        };

        var initTooltip = function(){
            var previousPoint = null;
            var $graph_data_tooltip = $('#graph_data_tooltip');

            $graph_data_tooltip.css({
                position: 'absolute',
                display: 'none',
                //border: '1px solid #666',
                'border-top-left-radius': '5px',
                'border-top-right-radius': '0',
                'border-bottom-left-radius': '0',
                'border-bottom-right-radius': '5px',
                padding: '2px 4px',
                'background-color': '#f2f2f2',
                'border-radius': '5px',
                opacity: 0.9,
                'box-shadow': '2px 2px 3px #888',
                transition: 'all 1s'
            });

            $('#graphCanvas').bind('plothover', function(event, pos, item){
                $('#x').text(pos.pageX.toFixed(2));
                $('#y').text(pos.pageY.toFixed(2));

                if(item){
                    if(previousPoint != item.dataIndex){
                        previousPoint = item.dataIndex;

                        $('#graph_data_tooltip').hide();

                        var value = item.datapoint[1];
                        if(!isNaN(value) && isFinite(value)){
                            value = value.toFixed(4);
                        }
                        var tooltip_text = value;
                        if($scope.currentGraphUnit){
                            tooltip_text += ' ' + $scope.currentGraphUnit;
                        }

                        showTooltip(item.pageX, item.pageY, tooltip_text, item.datapoint[0]);
                    }
                }else{
                    $("#graph_data_tooltip").hide();
                    previousPoint = null;
                }
            });
        };

        var showTooltip = function(x, y, contents, timestamp){
            var $graph_data_tooltip = $('#graph_data_tooltip');

            var date = luxon.DateTime.fromJSDate(new Date(timestamp)).setZone($scope.timezone.user_timezone);
            var humanTime = date.toFormat('dd.LL.yyyy HH:mm:ss');

            $graph_data_tooltip
                .html('<i class="fa fa-clock-o"></i> ' + humanTime + '<br /><b>' + contents + '</b>')
                .css({
                    top: y,
                    left: x + 10
                })
                .appendTo('body')
                .fadeIn(200);
        };

        var renderGraph = function(performance_data){
            initTooltip();

            var thresholdLines = [];
            var thresholdAreas = [];

            var GraphDefaultsObj = new GraphDefaults();

            var defaultColor = GraphDefaultsObj.defaultFillColor;
            if(performance_data.datasource.warn !== "" &&
                performance_data.datasource.crit !== "" &&
                performance_data.datasource.warn !== null &&
                performance_data.datasource.crit !== null){

                var warn = parseFloat(performance_data.datasource.warn);
                var crit = parseFloat(performance_data.datasource.crit);

                //Add warning and critical line to chart
                thresholdLines.push({
                    color: GraphDefaultsObj.warningBorderColor,
                    yaxis: {
                        from: warn,
                        to: warn
                    }
                });

                thresholdLines.push({
                    color: GraphDefaultsObj.criticalBorderColor,
                    yaxis: {
                        from: crit,
                        to: crit
                    }
                });

                //Change color of the area chart for warning and critical
                if(warn > crit){
                    defaultColor = GraphDefaultsObj.okFillColor;
                    thresholdAreas.push({
                        below: warn,
                        color: GraphDefaultsObj.warningFillColor
                    });
                    thresholdAreas.push({
                        below: crit,
                        color: GraphDefaultsObj.criticalFillColor
                    });
                }else{
                    defaultColor = GraphDefaultsObj.criticalFillColor;
                    thresholdAreas.push({
                        below: crit,
                        color: GraphDefaultsObj.warningFillColor
                    });
                    thresholdAreas.push({
                        below: warn,
                        color: GraphDefaultsObj.okFillColor
                    });
                }
            }

            var graph_data = [];
            for(var timestamp in performance_data.data){
                graph_data.push([timestamp, performance_data.data[timestamp]]);
            }

            var options = GraphDefaultsObj.getDefaultOptions();

            options.height = '300';

            options.colors = defaultColor;
            options.tooltip = true;
            options.tooltipOpts = {
                defaultTheme: false
            };
            options.xaxis.tickFormatter = function(val, axis){
                var date = luxon.DateTime.fromJSDate(new Date(val)).setZone($scope.timezone.user_timezone);
                return date.toFormat('dd.LL.yyyy HH:mm:ss');
            };
            options.series.color = defaultColor;
            options.series.threshold = thresholdAreas;
            options.grid.markings = thresholdLines;
            //options.lines.fillColor.colors = [{opacity: 0.4}, {brightness: 1, opacity: 1}];
            options.lines.fillColor.colors = [{brightness: 1, opacity: 0.2}, {brightness: 1, opacity: 0.2}];

            //options.points = {
            //    show: $scope.graph.showDatapoints,
            //    radius: 2.5
            //};


            options.xaxis.min = (lastGraphStart * 1000);
            options.xaxis.max = (graphRenderEnd * 1000);

            options.yaxis.axisLabel = performance_data.datasource.unit;

            //$scope.currentGraphUnit = null;
            //if(performance_data.datasource.unit){
            //    $scope.currentGraphUnit = performance_data.datasource.unit;
            //}

            if($scope.graph.smoothInterpolation === true){
                // Enable curved lines
                plot = $.plot('#graphCanvas', [{
                    data: graph_data, // Pass data for curved lines
                    // https://github.com/MichaelZinsmaier/CurvedLines
                    curvedLines: {
                        apply: true,
                        monotonicFit: true,
                        //tension: 1
                    },
                    points: {
                        show: false
                    },
                },
                    {
                        //original data points
                        data: graph_data,
                        points: {
                            show: $scope.graph.showDatapoints,
                            radius: 2.5
                        },
                        lines: {
                            show: false
                        }
                    }
                ], options);
            }else{
                // Use default flot chart
                plot = $.plot('#graphCanvas', [{
                    data: graph_data,
                    curvedLines: {
                        apply: false
                    },
                    points: {
                        show: $scope.graph.showDatapoints,
                        radius: 2.5
                    },
                }], options);
            }

            if(zoomCallbackWasBind === false){
                $("#graphCanvas").bind("plotselected", function(event, ranges){
                    var start = parseInt(ranges.xaxis.from / 1000, 10);
                    var end = parseInt(ranges.xaxis.to / 1000, 10);


                    //Zoomed from right to left?
                    if(start > end){
                        var tmpStart = end;
                        end = start;
                        start = tmpStart;
                    }

                    var currentTimestamp = Math.floor($scope.serverTimeDateObject.getTime() / 1000);
                    var graphAutoRefreshIntervalInSeconds = $scope.graphAutoRefreshInterval / 1000;

                    //Only enable autorefresh, if graphEnd timestamp is near to now
                    //We dont need to autorefresh data from yesterday
                    if((end + graphAutoRefreshIntervalInSeconds + 120) < currentTimestamp){
                        disableGraphAutorefresh();
                    }
                    loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, false, start, end, true);
                });
            }

            zoomCallbackWasBind = true;
        };

        var enableGraphAutorefresh = function(){
            $scope.graph.graphAutoRefresh = true;

            if(graphAutoRefreshIntervalId === null){
                graphAutoRefreshIntervalId = $interval(function(){
                    //Find last timestamp to only load new data and keep the existing
                    var lastTimestampInCurrentData = 0;
                    for(var timestamp in $scope.perfdata.data){
                        timestamp = parseInt(timestamp, 10);
                        if(timestamp > lastTimestampInCurrentData){
                            lastTimestampInCurrentData = timestamp;
                        }
                    }

                    // Get back to server time
                    var start = lastTimestampInCurrentData / 1000;

                    $scope.serverTimeDateObject = new Date($scope.serverTimeDateObject.getTime() + $scope.graphAutoRefreshInterval);

                    var end = Math.floor($scope.serverTimeDateObject.getTime() / 1000);
                    if(start > 0){
                        loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, true, start, end, false);
                    }
                }, $scope.graphAutoRefreshInterval);
            }
        };

        var disableGraphAutorefresh = function(){
            $scope.graph.graphAutoRefresh = false;

            if(graphAutoRefreshIntervalId !== null){
                $interval.cancel(graphAutoRefreshIntervalId);
            }
            graphAutoRefreshIntervalId = null;
        };


        /**** "Public" Methods ****/

        $scope.showFlashMsg = function(){
            new Noty({
                theme: 'metroui',
                type: 'success',
                layout: 'topCenter',
                text: $scope.flashMshStr,
                timeout: 4000
            }).show();

            $scope.showFlashSuccess = true;
            $scope.autoRefreshCounter = 5;
            var interval = $interval(function(){
                $scope.autoRefreshCounter--;
                if($scope.autoRefreshCounter === 0){
                    $scope.load();
                    $interval.cancel(interval);
                    $scope.showFlashSuccess = false;
                }
            }, 1000);
        };

        $scope.load = function(){

            $q.all([
                $http.get("/services/browser.json", {
                    params: {
                        angular: true,
                        serviceuuid: $stateParams.serviceuuid
                    }
                }), $http.get("/angular/user_timezone.json", {
                    params: {
                        angular: true
                    }
                })
            ]).then(function(results){
                $scope.servicestatus = results[0].data.servicestatus;
                $scope.acknowledgement = results[0].data.acknowledgement;
                $scope.downtime = results[0].data.downtime;

                $scope.graphAutoRefreshInterval = $scope.servicestatus.normal_check_interval * 1000;
                $scope.timezone = results[1].data.timezone;
                $scope.serverTimeDateObject = new Date($scope.timezone.server_time_iso);
                //$scope.serverTimeDateObject = luxon.DateTime.fromISO($scope.timezone.server_time_iso).setZone($scope.timezone.user_timezone);
                graphStart = (parseInt($scope.serverTimeDateObject.getTime() / 1000, 10) - ($scope.currentSelectedTimerange * 3600));
                //graphStart = (parseInt($scope.serverTimeDateObject.ts / 1000, 10) - ($scope.currentSelectedTimerange * 3600));
                graphEnd = parseInt($scope.serverTimeDateObject.getTime() / 1000, 10);
                //graphEnd = parseInt($scope.serverTimeDateObject.ts / 1000, 10);

                $scope.dataSources = [];
                for(var dsName in results[0].data.Perfdata){
                    $scope.dataSources.push(dsName);
                }
                if($scope.dataSources.length > 0){
                    if($scope.currentDataSource === null){
                        $scope.currentDataSource = $scope.dataSources[0];
                    }

                    loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, false, graphStart, graphEnd, true);
                }

                $scope.init = false;
                $scope.loadHostStatus(); //Fires on load of page
            });


            //$http.get("/services/browser.json", {
            //    params: {
            //        serviceuuid: $stateParams.serviceuuid
            //    }
            //}).then(function(result){
            //    $scope.servicestatus = result.data.servicestatus;
            //    $scope.acknowledgement = result.data.acknowledgement;
            //    $scope.downtime = result.data.downtime;
//
            //    $scope.init = false;
            //    $scope.loadHostStatus(); //Fires on load of page
            //});
        };

        $scope.loadHostStatus = function(){
            $http.get("/hosts/browser.json", {
                params: {
                    uuid: $stateParams.hostuuid
                }
            }).then(function(result){
                $scope.hoststatus = result.data.hoststatus;
                $scope.hostAcknowledgement = result.data.acknowledgement;
                $scope.hostDowntime = result.data.downtime;

                $scope.init = false;
            });
        };

        $scope.getObjectsForDowntimeDelete = function(){
            return [{
                Host: {
                    uuid: $scope.servicestatus.hostname
                },
                Service: {
                    uuid: $scope.servicestatus.service_description
                },
                Downtime: {
                    internal_downtime_id: $scope.downtime.internal_downtime_id
                }
            }];
        };

        $scope.getObjectForHostDowntimeDelete = function(){
            return [{
                Host: {
                    uuid: $scope.hostDowntime.hostname
                },
                Downtime: {
                    internal_downtime_id: $scope.hostDowntime.internal_downtime_id
                }
            }];
        };

        $scope.changeGraphTimespan = function(timespan){
            $scope.currentSelectedTimerange = timespan;
            var start = parseInt(new Date(new Date($scope.timezone.server_time_iso)).getTime() / 1000, 10) - (timespan * 3600);
            var end = parseInt(new Date(new Date($scope.timezone.server_time_iso)).getTime() / 1000, 10);

            //graphTimeSpan = timespan;
            loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, false, start, end, true);
        };

        $scope.changeDataSource = function(gaugeName){
            $scope.currentDataSource = gaugeName;

            //Reset unit - new datasource new unit - maybe
            $scope.currentGraphUnit = null;
            loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, false, lastGraphStart, lastGraphEnd, false);
        };

        $scope.changeAggregation = function(aggregation){
            $scope.currentAggregation = aggregation;
            loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, false, lastGraphStart, lastGraphEnd, false);
        };

        //Disable status update interval, if the object gets removed from DOM.
        $scope.$on('$destroy', function(){
            disableGraphAutorefresh();
        });

        //Fire on page load
        $scope.load();

        if($("[rel=tooltip]").length){
            $("[rel=tooltip]").tooltip();
        }

        $scope.getObjectsForExternalCommand = function(){
            return [{
                Host: {
                    uuid: $scope.hoststatus.hostname
                },
                Service: {
                    uuid: $scope.servicestatus.service_description
                }
            }];
        };

        $scope.getServicestatusTextColor = function(){
            if($scope.init){
                return;
            }

            switch($scope.servicestatus.current_state){
                case 0:
                case '0':
                    return 'ok';

                case 1:
                case '1':
                    return 'warning';

                case 2:
                case '2':
                    return 'critical';

                case 3:
                case '3':
                    return 'secondary';
            }
            return 'txt-primary';
        };


        $scope.stateIsOk = function(){
            return parseInt($scope.servicestatus.current_state, 10) === 0;
        };

        $scope.stateIsWarning = function(){
            return parseInt($scope.servicestatus.current_state, 10) === 1;
        };

        $scope.stateIsCritical = function(){
            return parseInt($scope.servicestatus.current_state, 10) === 2;
        };

        $scope.stateIsUnknown = function(){
            return parseInt($scope.servicestatus.current_state, 10) === 3;
        };

        $scope.stateIsNotInMonitoring = function(){
            // return !$scope.servicestatus.isInMonitoring;
            return temporaryAlwaysInMonitoring = false;
        };

        $scope.$watch('graph.graphAutoRefresh', function(){
            if($scope.init){
                return;
            }

            if($scope.graph.graphAutoRefresh === true){
                enableGraphAutorefresh();
            }else{
                disableGraphAutorefresh();
            }
        });

        $scope.$watch('graph.showDatapoints', function(){
            if($scope.init){
                return;
            }
            loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, false, lastGraphStart, lastGraphEnd, false);
        });

        $scope.$watch('graph.smoothInterpolation', function(){
            if($scope.init){
                return;
            }

            if($scope.graph.smoothInterpolation){
                LocalStorageService.setItem('smoothGraphInterpolation', 'true');
            }else{
                LocalStorageService.setItem('smoothGraphInterpolation', 'false');
            }

            loadGraph($stateParams.hostuuid, $stateParams.serviceuuid, false, lastGraphStart, lastGraphEnd, false);
        });
    });
