<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;

/**
 * Class Launchd
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class Launchd extends BasePlugin implements PluginInterface {


    public function execute() {
        if ($this->checkError('launchd_services')) {
            return;
        }

        if (!isset($this->json['launchd_services']) || !is_array($this->json['launchd_services'])) {
            $this->setOutput('Key "launchd_services" not found in Agent output');
            return;
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $match = null;
        $strict = true;
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $match = trim($arg);
                    break;
                case 1:
                    $strict = (bool)$arg;
                    break;
            }
        }

        foreach ($this->json['launchd_services'] as $service) {
            if ($this->serviceLabelMatch($match, $service['Label'], $strict)) {
                if ($service['IsRunning']) {
                    $this->setStatuscode(NagiosService::STATUS_OK);
                    $this->setOutput(sprintf(
                        '%s is running (PID=%s)',
                        $service['Label'],
                        $service['Pid']
                    ));
                    return;
                } else {
                    $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                    $this->setOutput(sprintf(
                        '%s is stopped',
                        $service['Label']
                    ));
                    return;
                }
            }
            $this->setOutput(sprintf('Service (%s) not found', $match));
            $this->setStatuscode(NagiosService::STATUS_CRITICAL);
        }
    }

    private function serviceLabelMatch($match, $label, $strict) {
        if ($strict && $match === $label) {
            return true;
        }
        if (!$strict && strpos($label, $match) !== false) {
            return true;
        }
        return false;
    }
}
