<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class SystemLoad
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class SystemUptime extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('agent', 'system_uptime')) {
            return;
        }

        $requiredKeys = [
            'agent' => [
                'system_uptime'
            ]
        ];

        foreach ($requiredKeys as $mainKey => $subKeys) {
            foreach ($subKeys as $subKey) {
                if (!isset($this->json[$mainKey][$subKey])) {
                    $this->setOutput(sprintf('Key "%s.%s" not found in Agent output', $mainKey, $subKey));
                    $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                    return;
                }
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }


        // By default this plugin should only record the uptime but not have any thresholds
        $warning = '-1:';
        $critical = '-1:';
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $warning = $arg;
                    break;
                case 1:
                    $critical = $arg;
                    break;
            }
        }

        $this->addPerfdata(new PerfdataGauge(
            'uptime',
            $this->json['agent']['system_uptime'],
            's',
            $warning,
            $critical,
            0
        ));

        $timeAgoInWords = $this->json['agent']['system_uptime'];
        try{
            $timeAgoInWords = $this->secondsInHumanShort($this->json['agent']['system_uptime']);
        }catch (\Exception $e){
            // Ignore any errors
        }

        $this->setOutput(sprintf(
            'System is up since: %s. Uptime: %s',
            date('d.m.Y H:i:s', (time() - $this->json['agent']['system_uptime'])),
            $timeAgoInWords
        ));


        if ($this->getRangeStatus($critical, $this->json['agent']['system_uptime'])) {
            $this->setStatuscode(NagiosService::STATUS_CRITICAL);
            return;
        }

        if ($this->getRangeStatus($warning, $this->json['agent']['system_uptime'])) {
            $this->setStatuscode(NagiosService::STATUS_WARNING);
            return;
        }

        $this->setStatuscode(NagiosService::STATUS_OK);
    }

}
